/*********************************************************************
 *                
 * Filename:      driver.h
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Haris Zukanovic <haris@stud.cs.uit.no>
 * Created at:    Tue Apr 14 12:41:42 1998
 * Modified at:   Tue Oct 20 10:30:57 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998 Haris Zukanovic, All Rights Reserved.
 *     Copyright (c) 1998 Dag Brattli, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Haris Zukanovic nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifndef DRIVER_H
#define DRIVER_H

#include <linux/tty.h>

#include "irda.h"
#include "qos.h"
#include "irqueue.h"

struct buff_t {
	int magic;

	int state;              /* Receiving state (transmit state not used) */
	int in_frame;           /* True if receiving frame */

	__u8 *head;		/* where in the buffer we are */
	__u8 *buff;		/* the buffer */

	int ptr;		/* location of pointer into buffer */
	int count;		/* currently used bytes in buffer */
	int buffsize;		/* size of the buffer */
	__u16 fcs;
	
	/*  
	 *  And the list of outstanding skb's for device 
	 */
	struct sk_buff_head list;
};

#define SIR_MODE 	0x01
#define MIR_MODE 	0x02
#define FIR_MODE 	0x04
#define IO_PIO   	0x08
#define IO_DMA		0x10

#define XMIT	0x01
#define RCV	0x02

#define BUF_MAX_LEN	4096

struct driver_cb {
	QUEUE queue;    /* Must be first */

        int magic;	/* our magic bullet */
	struct irlap_cb *irlap;
 	void *self;

	__u16 driver_flags;	/* driver capabilities/needs */
	__u16 flags;

	char *name;
	struct qos_info qos;  /* QoS capabilities for this device */
	struct buff_t tx;
	struct buff_t rx;
	int media_busy;
	
	/* Media busy stuff */
	struct timer_list media_busy_timer;
	struct timer_list todo_timer;


	void (*hard_xmit)( struct driver_cb *driver, struct sk_buff *skb);
        void (*change_speed)( struct driver_cb *driver, int baud);

/* 	int (*is_receiving)(struct driver_cb *driver); */  /* receiving? */
	int (*is_tbusy)(struct driver_cb *driver);	   /* transmitting? */

};

extern HASHBIN *driver;

/* Function prototypes */
int  driver_init( void);
void driver_cleanup( void);

int  driver_open( struct driver_cb *);
void driver_close( struct driver_cb *);

/* Interface to be uses by IrLAP */
inline int  driver_queue_xmit( struct driver_cb *, struct sk_buff *skb);
inline void driver_set_media_busy( struct driver_cb *, int status);
inline int  driver_is_media_busy( struct driver_cb *);
inline int  driver_is_receiving( struct driver_cb *);
inline void driver_change_speed( struct driver_cb *, int);

inline struct qos_info *driver_get_qos( struct driver_cb *self);

#endif







