/*********************************************************************
 *                
 * Filename:      esi.c
 * Version:       Driver for the Extended Systems JetEye PC
 * Description:   dongle driver
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Wed Sep 23 01:56:10 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:	  esi.c
 *
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>,
 *     Copyright (c) 1998, Dag Brattli,  <dagb@cs.uit.no>
 *     All Rights Reserved.
 *
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software.
 *     This material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

/* #include <linux/module.h> */

#include <linux/delay.h>
#include <linux/tty.h>
#include <linux/sched.h>
#include <asm/ioctls.h>

#include "ircompat.h"
/* make depend does not understand LinuxVersionCode(2,1,0) */
#if LINUX_VERSION_CODE > 0x020100
#include <asm/uaccess.h>
#endif

#include "irda.h"
#include "irmod.h"
#include "driver.h"
#include "irtty.h"

/*
 * Function esi_change_speed (tty, baud)
 *
 *    Set the speed for the Extended Systems JetEye PC ESI-9680 type dongle
 *
 */
void esi_change_speed( struct driver_cb *driver, int baud) {
	struct irtty_cb *self;
	struct tty_struct *tty;
	int arg = 0;
        struct termios old_termios;
	int cflag;
#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
	mm_segment_t fs;
#else
	unsigned long fs;
#endif
	
	DEBUG( 0, __FUNCTION__ "()\n");
	
	ASSERT( driver != NULL, return;);
	ASSERT( driver->magic == DRIVER_MAGIC, return;);
	
	self = (struct irtty_cb *) driver->self;
	
	ASSERT( self != NULL, return;);
	ASSERT( self->magic == IRTTY_MAGIC, return;);

	if ( self->tbusy || self->tty->driver.chars_in_buffer( self->tty)) {
		self->new_speed = baud;
		
		irda_start_todo_timer( self, 5); /* 50 msecs */
		return;
	}

	if ( !self->tty)
		return;

	tty = self->tty;
	
	old_termios = *(tty->termios);
	cflag = tty->termios->c_cflag;

	cflag &= ~CBAUD;

	switch (baud) {
	case 19200:
		cflag |= B19200;
		arg = TIOCM_DTR;
		break;
	case 115200:
		cflag |= B115200;
		arg = TIOCM_RTS | TIOCM_DTR;
		break;
	case 9600:
	default:
		cflag |= B9600;
		arg = TIOCM_RTS;
		break;
	}
		
	tty->termios->c_cflag = cflag;
	tty->driver.set_termios( tty, &old_termios);

	/*
	 *  The ioctl function, or actually set_modem_info in serial.c
	 *  expects a pointer to the argument in user space. To hack us
	 *  around this we use the set_fs function to fool the routines 
	 *  that check if they are called from user space. We also need 
	 *  to send a pointer to the argument so get_user() gets happy. 
	 *  DB.
	 */
	fs = get_fs();
	set_fs( get_ds());

	if ( tty->driver.ioctl( tty, NULL, TIOCMSET, (unsigned long) &arg)) { 
		DEBUG(0, "error setting ESI speed!\n");
	}
	set_fs(fs);

	self->new_speed = 0;
}

/*
 * Function init_module (void)
 *
 *    Initialize IrTTY module
 *
 */
/* int init_module(void) */
/* { */
/* 	irtty_init(); */
/* 	return(0); */
/* } */

/*
 * Function cleanup_module (void)
 *
 *    Cleanup IrTTY module
 *
 */
/* void cleanup_module(void) */
/* { */
/* 	DEBUG( 4, "IrTTY: cleanup_module!\n"); */
/* 	irtty_cleanup(); */
/* } */
