/*********************************************************************
 *                
 * Filename:      fir_skeleton.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Haris Zukanovic <haris@stud.cs.uit.no>
 * Created at:    Tue Apr 14 12:39:06 1998
 * Modified at:   Tue Apr 14 21:32:31 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1997 Haris Zukanovic, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Haris Zukanovic nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/
#include <linux/module.h>

#include "irvdd.h"
#include "fir_skeleton.h"

extern int irvdd_register_fir_device(struct fir_cb *);
extern int irvdd_unregister_fir_device(struct fir_cb *);

#define FIR_DRIVER_ID 0

#define FIR_DRIVER_NAME  "SKELETON"
#define FIR_VER "0.1"

char *ttyname=0;  /* this variable shall be initialized 
		     when loading module */

/* 
 *  FIR code 
 */

/* 
 *  NOTE (suggestion):
 *  Functions in this "skeleton" file are called 
 *  fir_function_name(tralala);
 *  It's recommended to call functions
 *  "device_name"_function_name(tralala);
 *   e.g. HDSL1100_change_speed(tralala);
 *  
 */

/*
 * Function fir_change_speed (irvdd, speed)
 *
 *    
 *
 */
int fir_change_speed(struct fir_cb *fir, int speed)
{
	/* 
	 *  Here you set up the hardware to send at desired speed 
	 *  and setup IrPHY_xmit() in the irvdd.
	 */

	return(0);
}

/*
 * Function fir_xmit (irvdd, skb)
 *
 *    
 *
 */
int fir_xmit(struct fir_cb * fir,struct sk_buff *skb)
{
	printk("fir: he he, irvdd_xmit called\n");
	return(0);
}

/*
 * Function fir_is_media_busy (irvdd)
 *
 *    
 *
 */
int fir_is_media_busy(struct fir_cb *fir)
{
	
	return(0);
}

/*
 * Function fir_is_receiving (irvdd)
 *
 *    
 *
 */
int fir_is_receiving(struct fir_cb *fir)
{
	return(FALSE);
}

/* 
 *  Data
 */

/*  If the driver supports more than one fir device there has to be one
 *  fir_cb for each device present in the system. Each one must be
 *  registered with Irvdd That's the only way they all can be utilized by
 *  irlap.  
 */
struct fir_cb irvdd_reg=
{
	FIR_DRIVER_ID,           /* fir driver id
				    Can be used if driver supports more than 
				    one device */
	SIR_MODE,
	FIR_DRIVER_NAME,         /* fir device name */
	0,                       /* tty name, set in init_module() */
	fir_xmit,                /* xmit routine   */
	fir_change_speed,        /* change_speed routine */
	fir_is_media_busy,       /* media busy routine */
	fir_is_receiving         /* receiving frame flag state */
};  

/* 
 *   Module stuff 
 */

/*
 * Function init_module (void)
 *
 *    
 *
 */
int init_module(void)
{
	if (ttyname==0)
	{
		printk( "%s: parameter <ttyname> NOT set, value is(%s), "
			"init failed.  (ttyname=ttySn)\n", 
			FIR_DRIVER_NAME, ttyname);
		return(-EINVAL);
	}
	irvdd_reg.tty_name=ttyname;
	
	/*
	 *  When irvdd_data is ready initialized register ourselves at irvdd 
	 *  (virtual device layer) 
	 */
	if (irvdd_register_fir_device(&irvdd_reg))
	{
		printk( "%s: %s driver ID %d failed registering with the "
		        "Irvdd layer on %s.\n",
			FIR_DRIVER_NAME,FIR_DRIVER_NAME,FIR_DRIVER_ID,ttyname);
		return(-1);
	}
	else
		printk("%s: %s FIR driver ID %d registered with the Irvdd "
		       "layer on %s.\n",
		       FIR_DRIVER_NAME,FIR_DRIVER_NAME,FIR_DRIVER_ID,ttyname);
  
	return(0);
}

/*
 * Function cleanup_module (void)
 *
 *    
 *
 */
void cleanup_module(void)
{
	/* relase tty and stuff */
	
	/* unregister ourselves at irvdd */
	irvdd_unregister_fir_device(&irvdd_reg);
	printk( "%s: %s FIR driver ID %d unregistered from the Irvdd "
		"layer on %s.\n",
		FIR_DRIVER_NAME,FIR_DRIVER_NAME,FIR_DRIVER_ID,ttyname);
}
