/*********************************************************************
 *                
 * Filename:      irport.h
 * Version:       0.1
 * Description:   Serial driver for IrDA
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sun Aug  3 13:49:59 1997
 * Modified at:   Tue Dec  2 12:07:35 1997
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1997 Dag Brattli <dagb@cs.uit.no>, All Rights Reserved.
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#if 0
static char *rcsid = "$Id: irport.h,v 1.1 1998/10/24 14:49:04 ryan Exp $";
#endif

#ifndef IRPORT_H
#define IRPORT_H

#include <linux/netdevice.h>
#include <linux/skbuff.h>
#include <linux/types.h>

#include "driver.h"

#define SPEED_DEFAULT 9600
#define SPEED_MAX     115200

/*
 * These are the supported serial types.
 */
#define PORT_UNKNOWN    0
#define PORT_8250       1
#define PORT_16450      2
#define PORT_16550      3
#define PORT_16550A     4
#define PORT_CIRRUS     5
#define PORT_16650      6
#define PORT_MAX        6  

#define FRAME_MAX_SIZE 2048

struct chipio_cb {
	int magic;

	int iobase;		/* IO base */  
	int ioextent;		/* # of io ports needed */
	int irq;		/* ACE irq */
	int fifo_size;		/* FIFO size */

	int dma;		/* DMA channel used */
	int irqflags;		/* interrupt flags (ie, SA_SHIRQ|SA_INTERRUPT) */
};

struct fir_driver_cb {
  
  /* Important Data */
  struct chipio_cb SCE;       /* Chip specific info */
  struct driver_cb drv;       /* General Driver */
	int baud;                   /* Current Baud Rate */
	int media_busy;             /* */
  int new_speed;              /* Set when we have to wait to set the baud */
  int tbusy;                  /* Busy transmitting flag */
  int sbusy;                  /* Busy changing speed flag */
  int direction;              /* Receiving or Transmiting (For FIR only) */
  
  /* Functions that need to be kept track of */
	void (*irq_handler) (int irq, void *dev_id, struct pt_regs *regs);

};

int  irport_init(void);
void irport_cleanup(struct fir_driver_cb *driver);
int  irport_start(struct fir_driver_cb *driver);
int  irport_detect(struct fir_driver_cb *driver);

void irport_setspeed( struct driver_cb *driver, int speed);

void irport_xmit(struct driver_cb *driver, struct sk_buff *skb);
int  irport_write(struct driver_cb *driver);
int irport_receive(struct fir_driver_cb *driver);

void irport_bh(void *data);
void irport_set_media_busy(struct driver_cb *driver,  int status);
int  irport_is_media_busy(struct driver_cb *driver);

void irport_async_unwrap( struct driver_cb *driver, __u8 byte);
int irport_async_wrap( struct sk_buff *skb, __u8 *tx_buff, int buffsize);

#endif
