/*********************************************************************
 *                
 * Filename:      irtty.h
 * Version:       0.1
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Tue Dec  9 21:13:12 1997
 * Modified at:   Tue Oct 20 09:23:57 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 *  
 *     Copyright (c) 1997 Dag Brattli, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifndef IRTTY_H
#define IRTTY_H

#include <linux/if.h>
#include <linux/skbuff.h>
#include <linux/termios.h>

#include <irda.h>
#include <irqueue.h>
#include <driver.h>

#define IRTTY_IOC_MAGIC 'e'

#define IRTTY_IOCTESI  _IO(IRTTY_IOC_MAGIC, 1)
#define IRTTY_IOC_MAXNR 1

#ifndef N_IRDA
#define N_IRDA         11     /* This one should go in .../asm/termio.h */
#endif

#define BUF_MAX_LEN    4096

struct irtty_cb {
	QUEUE *queue; /* Must be first */

	int	magic;
	int     tbusy;       /* Busy transmitting flag */
	
	struct  tty_struct *tty;  /* Ptr to TTY structure */
	struct  driver_cb driver;

	struct timer_list todo_timer;

	int new_speed;
	
	char name[10];
};
 
int  irtty_init(void);
void irtty_cleanup(void);
int  irtty_open( struct tty_struct *tty);
void irtty_close( struct tty_struct *tty);
int  irtty_ioctl( struct tty_struct *tty, void *file, int cmd, void *arg);
int  irtty_receive_room( struct tty_struct *tty);
void irtty_change_speed( struct driver_cb *self, int baud);
void irtty_write_wakeup( struct tty_struct *tty);
void irtty_queue_xmit( struct driver_cb *this, struct sk_buff *skb);
void irtty_xmit_buf( struct driver_cb *this, struct sk_buff *skb);
void irtty_receive_buf( struct tty_struct *tty, const unsigned char *cp, 
						char *fp, int count);
inline int irtty_is_tbusy( struct driver_cb *driver);

void irda_start_mbusy_timer( struct irtty_cb *irtty);
void irtty_media_busy_expired( unsigned long data);
void irda_start_todo_timer( struct irtty_cb *irtty, int timeout);
void irtty_todo_expired( unsigned long data);
int  irtty_is_receiving( struct driver_cb *self);

#endif
