/*********************************************************************
 *                
 * Filename:      generic.c
 * Version:       
 * Description:   dongle driver
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Sun Apr  5 01:08:41 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:	  geeric.c
 *
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>,
 *			   Dag Brattli,  <dagb@cs.uit.no>
 *     All Rights Reserved.
 *
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software.
 *     This material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/miscdevice.h>

#include <linux/tty.h>
#include <linux/serial.h>
#include <asm/io.h>

#include "irda.h"
#include "dongle.h"
#include "smc_ircc.h"
#include "qos.h"

void generic_setspeed(struct tty_struct *tty, int baud);

extern int esi_reset(struct tty_struct *tty);
extern void esi_setspeed(struct tty_struct *tty, int baud);

extern int ircc_init(struct tty_struct *tty);
extern void ircc_shutdown(struct tty_struct *tty);
extern void ircc_setspeed(struct tty_struct *tty, int baud);
extern void ircc_send(struct tty_struct *tty, struct sk_buff *skb);
extern void ircc_receive(struct tty_struct *tty, struct sk_buff *skb);

#if 0
static char *rcsid = "$Id: generic.c,v 1.1.1.1 1998/09/29 02:27:10 root Exp $";
#endif

struct driver_cb dongle_driver[] = {
        { IR_9600|IR_19200|IR_38400|IR_57600|IR_115200,
	  NULL,
	  NULL,
	  generic_setspeed,
	  NULL,
	  "generic"
	},
        { IR_9600,
	  NULL,
	  NULL,
	  generic_setspeed,
	  NULL,
	  "generic-unsupported"
	},
	{ IR_9600|IR_19200|IR_115200,
	  NULL,
	  NULL,
	  esi_setspeed,
	  NULL,
	  "esi"
	},
        { IR_9600|IR_19200|IR_38400|IR_57600|IR_115200|IR_576000|IR_1152000|(IR_4000000 << 8),
	  NULL,
	  NULL,
	  NULL,
	  NULL,
	  "fir"
	},

#if 0
#if 1
        { IR_9600|IR_19200|IR_38400|IR_57600|IR_115200|IR_576000|IR_1152000|(IR_4000000 << 8),
#else
#if 0
        { IR_9600|IR_19200|IR_38400|IR_57600|IR_115200|IR_576000|IR_1152000,
#else
        { IR_9600|IR_19200|IR_38400|IR_57600|IR_115200,
#endif
#endif
	  ircc_init,
	  ircc_shutdown,
	  ircc_setspeed,
	  ircc_send,
	  "smc_ircc"
	},
#endif
#if 0
	  { IR_9600|IR_19200|IR_38400|IR_57600|IR_115200|IR_576000|IR_1152000|(IR_4000000 << 8),
	  ibm_init,
	  ibm_shutdown,
	  ibm_setspeed,
	  ibm_send,
	  "ibm_fir"
	},
#endif
};


void generic_setspeed(struct tty_struct *tty, int baud)
{
        struct termios old_termios;
	int cflag;

	old_termios = *(tty->termios);
	cflag = tty->termios->c_cflag;

	cflag &= ~CBAUD;

	switch(baud) {
	case 1200:
		cflag |= B1200;
		break;
	case 2400:
		cflag |= B2400;
		break;
	case 4800:
		cflag |= B4800;
		break;
	case 19200:
		cflag |= B19200;
		break;
	case 38400:
		cflag |= B38400;
		break;
	case 57600:
		cflag |= B57600;
		break;
	case 115200:
		cflag |= B115200;
		break;
	case 9600:
	default:
		cflag |= B9600;
		break;
	}	

	tty->termios->c_cflag = cflag;
	tty->driver.set_termios( tty, &old_termios);
}
