/*********************************************************************
 *                
 * Filename:      irattach.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sun Dec  7 23:21:05 1997
 * Modified at:   Wed Oct 21 14:23:35 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:       
 *
 *     Copyright (c) 1997 Dag Brattli <dagb@cs.uit.no>, All Rights Reserved.
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <syslog.h>
#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>

#ifndef N_IRDA
#define N_IRDA 11 /* This one should go in .../asm/termio.h */
#endif

#define VERSION "0.1 Fri Jul 25 11:45:26 1997 Dag Brattli"

static int initfdflags = -1;	/* Initial file descriptor flags */
static int initdisc = -1;

#define IRTTY_IOC_MAGIC 'e'

#define IRTTY_IOCTESI  _IO(IRTTY_IOC_MAGIC, 1)
#define IRTTY_IOC_MAXNR 1

/*
 * Function establish_irda (fd)
 * 
 *    Turn the serial port into a irda interface.
 */
void establish_irda ( int fd) 
{
	int irdadisc = N_IRDA;
	
	if ( ioctl( fd, TIOCEXCL, 0) < 0) {
		syslog ( LOG_WARNING, "ioctl(TIOCEXCL): %m");
	}
	
	if (ioctl(fd, TIOCGETD, &initdisc) < 0) {
		syslog( LOG_ERR, "ioctl(TIOCGETD): %m");
		exit (1);
	}
	
	if (ioctl(fd, TIOCSETD, &irdadisc) < 0){
		fprintf( stderr,  
			 "Maybe you don't have IrDA support in your kernel?\n");
		syslog( LOG_ERR, "irattach: tty: set_disc(%d): %s\n", 
			irdadisc, strerror(errno));
		exit (1);
	}
}

/*
 * Function tty_configure (tios)
 *
 *    Put a IrDA line discipline in a transparent mode. 
 *
 */
static int tty_configure( struct termios *tios) 
{
	tios->c_cflag     = CS8 | CREAD | B9600 | CLOCAL;
	
	/* Ignore break condition and parity errors */
 	tios->c_iflag     = IGNBRK | IGNPAR;
	tios->c_oflag     = 0;
	tios->c_lflag     = 0; /* set input mode (non-canonical, no echo,..) */
	tios->c_cc[VMIN]  = 1; /* num of chars to wait for,before delivery */
	tios->c_cc[VTIME] = 0; /* timeout before delivery */
	
	return(0);
}

/*
 * Function init_irda (fd)
 *
 *    Initialize IrDA line discipline
 *
 */
void init_irda( int fd) 
{
	struct termios tios;
	
	tty_configure( &tios);
	
	/* tcflush(fd, TCIFLUSH); */
	if ( tcsetattr( fd, TCSAFLUSH, &tios) < 0) {
		syslog( LOG_ERR, "tcsetattr: %m");
		exit(1);
	}
}

/*
 * Function main (argc, )
 *
 *    Main function
 *
 */
int main( int argc, char *argv[]) 
{
	int fd = -1;
	int c;
	int dongle = 0;


	printf( "%s\n", VERSION);
	if( argc < 2) {
		printf( "Usage: %s /dev/<irda-port> [-d dongle]\n", argv[0]);
		exit( -1);
	}      
	
	/*
	 * Open the serial device and set it up to be the irda interface.
	 */
	if (( fd = open( argv[1], O_NONBLOCK | O_RDWR, 0)) < 0) {
		syslog( LOG_ERR, "Failed to open %s: %m", argv[1]);
		exit(1);
	}
	if (( initfdflags = fcntl( fd, F_GETFL)) == -1) {
		syslog( LOG_ERR, "Couldn't get device fd flags: %m");
		exit(1);
	}
	
	initfdflags &= ~O_NONBLOCK;
	fcntl( fd, F_SETFL, initfdflags);
	
	syslog( LOG_INFO, "Serial connection established.");
	
	/* 
	 *  Set up the serial device as a irda interface 
	 */
	
	init_irda( fd);
	establish_irda( fd);

	sleep(1);  /* give it time to set up its terminal */
	
	/*
	 *  Set device for non-blocking reads.
	 */
	if ( fcntl( fd, F_SETFL, initfdflags | O_NONBLOCK) == -1) {
		syslog( LOG_ERR, 
			"Couldn't set device to non-blocking mode: %m");
		exit(1);
	}
	
	c = getopt( argc, argv, "d:");
	if ( c == 'd') {
		/* Check for ESI-9680 type dongle */
		if ( strcmp( optarg, "ESI") == 0) {
			ioctl( fd, IRTTY_IOCTESI, dongle);
		} else {
			printf( "Sorry, dongle not supported yet!\n");
			exit( -1);
		}
	}

	/*
	 *  Loop forever and wait for kill or ctrl-C since closing this 
	 *  process will also close all open files for this process
	 *  which will in turn close the tty used for IrDA which is not
	 *  really what we want :-)
	 */
	while(1) 
		sleep(2);
	return 0;
}
