#! /usr/bin/perl 
######################################################################
##                
## Filename:      ircp.pl
## Version:       
## Description:   Linux/IR control panel
## Status:        Experimental.
## Author:        Dag Brattli <dagb@cs.uit.no>
## Created at:    Thu Jul  9 09:45:51 1998
## Modified at:   Fri Oct 16 14:51:51 1998
## Modified by:   Dag Brattli <dagb@cs.uit.no>
## 
##     Copyright (c) 1998 Dag Brattli, All Rights Reserved.
##      
##     This program is free software; you can redistribute it and/or 
##     modify it under the terms of the GNU General Public License as 
##     published by the Free Software Foundation; either version 2 of 
##     the License, or (at your option) any later version.
##  
##     Neither Dag Brattli nor University of Troms admit liability nor
##     provide warranty for any of this software. This material is 
##     provided "AS-IS" and at no charge.
##     
######################################################################

use Gtk;
use Gtk::Atoms;
use Gtk::Gnome;

init Gtk::Gnome "App_id";

$app = new Gtk::Gnome::App "ircp", "IrDA Control Panel";

#$about = new Gtk::Gnome::About "test";

##
## Function toggle_snap ()
##
##    
##
##
sub toggle_snap {
  my($widget, $spin) = @_;
  
  if ($widget->active) {
    $spin->set_update_policy(['always', 'snap_to_ticks']);
  } else {
    $spin->set_update_policy('always');
  }
}

##
## Function toggle_numeric ()
##
##    
##
##
sub toggle_numeric {
    my($widget, $spin) = @_;
    $spin->set_numeric($widget->active);
}

##
## Function get_value ()
##
##    
##
##
sub get_value {
    my($widget,$data) = @_;
    my($labels, $spin, $buf);
    
    $spin = $spinner1;
    $label = $widget->{label};
    if ($data == 1) {
	$buf = sprintf "%d", $spin->get_value_as_int;
    } else {
	$buf = sprintf "%0.*f", $spin->digits, $spin->get_value_as_float;
    }
    $label->set($buf);
}

##
## Function discovery_timeout ()
##
##    
##
##
sub discovery_timeout {
    my($label) = @_;
    
    open (DISCOVERY, "/proc/net/irda/discovery") || return;
    
    while ( $line = <DISCOVERY>) {
	chomp( $line);
	
	if ( $line =~ /Name:/) {
	    $line =~ s/Name://;
	    $name->set( $line);
	}
	elsif ( $line =~ /Hint:/) {
	    $line =~ s/Hint://;
	    $service->set( $line);
	} 
	elsif ( $line =~ /Dest/) {
	    $line =~ s/Dest addr://;
	    $daddr->set( $line);
	}
    }
    
    close (DISCOVERY) || die "can't close file: $!";
    
    return 1;
}

##
## Function progress_timeout ()
##
##    
##
##
sub progress_timeout {
    my(($progressbar, $baudrate)) = @_;
    my($new_val);
    
    open (IRLAP, "/proc/net/irda/irlap") || return;
    
    while ( <IRLAP>) {
	chomp;
	
	# 
	# If line starts with "retrans"
	#
	if ( /^retrans:/) {
	    ($dummy,$retrans,$vs,$vr,$va)= split(/\D*:/);
	}
	if ( /^session/) {
	    ($dummy, $bps,$maxtt,$dsize,$winsize,$addbofs,$mintt,$ldisc) = 
	      split;
	}
    }
    #printf "Retrans=%d, vs=%d, vr=%d, va=%d\n", $retrans, $vs, $vr, $va;
    
    $new_val = 1 - $retrans/6;
    $progressbar->update($new_val);
    
    $baudrate->set( $bps);

    close (IRLAP) || die "can't close file: $!";
    
    return 1;
}

sub create_menus {
    #
    #  Menubar
    #
    
    $menubar = new Gtk::MenuBar;
    $main_vbox->pack_start($menubar, 0, 1, 0);
    $menubar->show;
    
    $menu = new Gtk::Menu;
    $menuitem = new Gtk::MenuItem("Quit");
    $menu->append($menuitem);
    $menuitem->show;
    
    $menubaritem = new Gtk::MenuItem("File");
    $menubaritem->set_submenu($menu);
    $menubar->append($menubaritem);
    
    #    show $menuitem;    
    
    return $menubar;
}

##
## Function create_main_window ()
##
##    
##
##
sub create_main_window {
#    $window = new Gtk::Window('toplevel');
#    $window->set_name("main window");
#    $window->set_uposition(20, 20);
    #  $window->set_usize(200, 400);
    
#    $window->signal_connect("destroy" => \&Gtk::main_quit);
#    $window->signal_connect("delete_event" => \&Gtk::false);
    
#    $window->set_title("Linux/IR control panel");
#    $window->border_width(0);
    
    $main_vbox = new Gtk::VBox 0, 0;
    border_width $main_vbox 0;
#    $window->add($main_vbox);
    
    $menubar = create_menus;
    $main_vbox->pack_start($menubar,1,1,0);
    

    $frame_vbox = new Gtk::VBox 0, 0;
    $frame_vbox->border_width(10);
    $main_vbox->add($frame_vbox);
    
    #
    #  Connection frame
    #    
    $frame = new Gtk::Frame "Connection";
    $frame_vbox->pack_start($frame, 1, 1, 0);
    
    $vbox = new Gtk::VBox 0, 0;
    $vbox->border_width(5);
    $frame->add($vbox);
    
    $hbox = new Gtk::HBox(0,0);
    $vbox->pack_start($hbox, 1, 1, 5);
    
    $label = new Gtk::Label "Link status :";
    $label->set_alignment(0, 0.5);
    $hbox->pack_start($label, 0, 1, 0);
    
    #  $adj = new Gtk::Adjustment 1.0, 1.0, 31.0, 1.0, 5.0, 0.0;
    
    $pbar = new Gtk::ProgressBar;
    $pbar->set_usize(200,20);
    $hbox->pack_start($pbar,1,1,0);
    $pbar->show;
   
    $hbox = new Gtk::HBox 0, 0;
    $vbox->pack_start($hbox, 1, 1, 5);
    
    $label = new Gtk::Label "Speed :";
    $label->set_alignment(0, 0.5);
    $hbox->pack_start($label, 0, 1, 0);
    
    $speed = new Gtk::Label "<speed>";
    $speed->set_alignment(0, 0.5);
    $hbox->pack_start($speed, 0, 1, 0);
    
    $progress_timer = Gtk->timeout_add(1000, \&progress_timeout, 
				       ($pbar, $speed));
 
    #
    #  Discovery frame
    #    
    $frame = new Gtk::Frame "Discovery";
    $frame_vbox->pack_start($frame, 1, 1, 0);
    
    $vbox = new Gtk::VBox 1, 0;
    $vbox->border_width(5);
    $frame->add($vbox);
    
    $hbox = new Gtk::HBox 1, 0;
    $vbox->pack_start($hbox, 0, 1, 5);
    
    $label = new Gtk::Label "Name :";
    $label->set_alignment(0, 0.5);
    $hbox->pack_start($label, 0, 1, 0);
    
    $name = new Gtk::Label "<name>";
    $name->set_alignment(0, 0.5);
    $hbox->pack_start($name, 0, 1, 0);
    
    #  $adj = new Gtk::Adjustment 0.0, -10000.0, 10000.0, 0.5, 100.0, 0.0;
    
    $hbox = new Gtk::HBox 1, 0;
    $vbox->pack_start($hbox, 0, 1, 5);
    
    $label = new Gtk::Label "Service:";
    $label->set_alignment(0, 0.5);
    $hbox->pack_start($label, 0, 1, 0);
    
    $service = new Gtk::Label "<service>";
    $service->set_alignment(0, 0.5);
    $hbox->pack_start($service, 0, 1, 0);
    
    $discovery_timer = Gtk->timeout_add(1000, \&discovery_timeout, $label);
    
    $hbox = new Gtk::HBox 1, 0;
    $vbox->pack_start($hbox, 1, 1, 5);
    
    $label = new Gtk::Label "Dest addr:";
    $label->set_alignment(0, 0.5);
    $hbox->pack_start($label, 0, 1, 0);
    
    $daddr = new Gtk::Label "<dest addr>";
    $daddr->set_alignment(0, 0.5);
    $hbox->pack_start($daddr, 0, 1, 0);
    
    $hbox = new Gtk::HBox 0, 5;
    $frame_vbox->pack_start($hbox, 0, 1, 0);
    $hbox->border_width(5);
    
    $button = new Gtk::Button "Close";
#    $button->signal_connect(clicked => sub {destroy $window});
    $hbox->pack_start($button, 1, 1, 5);
    
#    show_all $window;

    show $main_vbox

    return $main_vbox;
 # $window->show;
}

#parse Gtk::Rc "testgtkrc";

$vbox = create_main_window;

$app->set_contents($vbox);
#$app->set_menus($menubar);

show_all $app;

main Gtk;
