/*********************************************************************
 *                
 * Filename:      irlan_common.c
 * Version:       0.1
 * Description:   IrDA LAN Access Protocol Implementation
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sun Aug 31 20:14:37 1997
 * Modified at:   Sun May 24 14:26:19 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:       skeleton.c by Donald Becker <becker@CESDIS.gsfc.nasa.gov>
 *                slip.c by Laurence Culhane,   <loz@holmes.demon.co.uk>
 *                          Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
 * 
 *     Copyright (c) 1997 Dag Brattli <dagb@cs.uit.no>, All Rights Reserved.
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include <linux/kernel.h>
#include <linux/string.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/byteorder.h>
#include <linux/errno.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>

#include <irda.h>
#include <ircompat.h>
#include <irttp.h>
#include <irlmp.h>
#include <iriap.h>
#include <timer.h>

#include "irlan_common.h"

/*
 * Function set_multicast_list (dev)
 *
 *    
 *
 */
void set_multicast_list( struct device *dev) 
{
/* 	struct irlan_cb *self; */

/* 	self = dev->priv; */

/* 	ASSERT( self != NULL, return;); */
/* 	ASSERT( self->magic == LAN_MAGIC, return;); */

	if (dev->mc_count || (dev->flags & IFF_ALLMULTI)) {
		/* Multicast Mode */
		DEBUG( 0, "Multicast mode\n");
		/* irlan_open_broadcast_addr( self); */
	} else if (dev->flags & IFF_PROMISC) {
		/* Promiscous mode */
		DEBUG( 0, "Promiscous mode\n");
	} else {
		/* Normal mode */
		DEBUG( 0, "Normal mode\n");
		/* irlan_open_unicast_addr( self); */
/* 		irlan_open_broadcast_addr( self); */
	}
}


/*
 * Function insert_byte (skb, param, value)
 *
 *    
 *
 */
int insert_byte( struct sk_buff *skb, char *param, __u8 value)
{
	__u8 *frame;
	__u8 len_param;
	__u16 len_value;
	int n=0;
	
	if ( skb == NULL) {
		DEBUG( 0, "insert_param: Got NULL skb\n");
		return 0;
	}
	
	len_param = strlen( param);
	len_value = 1;

	/*
	 *  Insert at end of sk-buffer
	 */
	frame = skb->tail;

	/* Make space for data */
	if ( skb_tailroom(skb) < (len_param+len_value+3)) {
		DEBUG( 0, "insert_param: No more space at end of skb\n");
		return 0;
	}	
	skb_put( skb, len_param+len_value+3);

	/* Insert parameter length */
	frame[n++] = len_param;
	
	/* Insert parameter */
	memcpy( frame+n, param, len_param); 
	n += len_param;
	
	/* Insert value length ( 2 byte little endian format, LSB first) */
	frame[n++] = len_value & 0xff;
	frame[n++] = len_value >> 8;

	frame[n++] = value;
	
	return len_param+len_value+3;
}

/*
 * Function insert_string (skb, param, value)
 *
 *    
 *
 */
int insert_string( struct sk_buff *skb, char *param, char *value)
{
	__u8 *frame;
	__u8 len_param;
	__u16 len_value;
	int n=0;
	
	if ( skb == NULL) {
		DEBUG( 0, "insert_param: Got NULL skb\n");
		return 0;
	}	
	len_param = strlen( param);
	len_value = strlen( value);

	/*
	 *  Insert at end of sk-buffer
	 */
	frame = skb->tail;

	/* Make space for data */
	if ( skb_tailroom(skb) < (len_param+len_value+3)) {
		DEBUG( 0, "insert_param: No more space at end of skb\n");
		return 0;
	}	
	skb_put( skb, len_param+len_value+3);

	/* Insert parameter length */
	frame[n++] = len_param;
	
	/* Insert parameter */
	memcpy( frame+n, param, len_param); 
	n += len_param;
	
	/* Insert value length ( 2 byte little endian format, LSB first) */
	frame[n++] = len_value & 0xff;
	frame[n++] = len_value >> 8;

	memcpy( frame+n, value, len_value); 
	n+=len_value;
	
	return len_param+len_value+3;
}

/*
 * Function insert_array (skb, param, value, len_value)
 *
 *    
 *
 */
int insert_array( struct sk_buff *skb, __u8 *param, __u8 *value, 
		  __u16 len_value)
{
	__u8 *frame;
	__u8 len_param;
	int n=0;
	
	if ( skb == NULL) {
		DEBUG( 0, "insert_param: Got NULL skb\n");
		return 0;
	}	
	len_param = strlen( param);

	/*
	 *  Insert at end of sk-buffer
	 */
	frame = skb->tail;

	/* Make space for data */
	if ( skb_tailroom(skb) < (len_param+len_value+3)) {
		DEBUG( 0, "insert_param: No more space at end of skb\n");
		return 0;
	}	
	skb_put( skb, len_param+len_value+3);

	/* Insert parameter length */
	frame[n++] = len_param;
	
	/* Insert parameter */
	memcpy( frame+n, param, len_param); 
	n += len_param;
	
	/* Insert value length ( 2 byte little endian format, LSB first) */
	frame[n++] = len_value & 0xff;
	frame[n++] = len_value >> 8;

	memcpy( frame+n, value, len_value); 
	n+=len_value;
	
	return len_param+len_value+3;
}

/*
 * Function insert_param (skb, param, value, byte)
 *
 *    Insert parameter at end of buffer, structure of a parameter is:
 *
 *    -----------------------------------------------------------------------
 *    | Name Length[1] | Param Name[1..255] | Val Length[2] | Value[0..1016]|
 *    -----------------------------------------------------------------------
 */
int insert_param( struct sk_buff *skb, 
		  char *param,
		  int type,
		  char *value_char, 
		  __u8 value_byte, 
		  __u16 value_short) 
{
	__u8 *frame;
	__u8 len_param;
	__u16 len_value;
	int n;
	
	if ( skb == NULL) {
		DEBUG( 0, "insert_param: Got NULL skb\n");
		return 0;
	}
	
	n = 0;

	len_param = strlen( param);
	switch ( type) {
	case 1:
		ASSERT( value_char != NULL, return 0;);
		len_value = strlen( value_char);
		break;
	case 2:
		len_value = 1;
		break;
	case 3:
		len_value = 2;
		break;
	default:
		DEBUG( 0, "Error in insert_param!\n");
		return 0;
		break;
	}
	
	/*
	 *  Insert at end of sk-buffer
	 */
	frame = skb->tail;

	/* Make space for data */
	if ( skb_tailroom(skb) < (len_param+len_value+3)) {
		DEBUG( 0, "insert_param: No more space at end of skb\n");
		return 0;
	}	
	skb_put( skb, len_param+len_value+3);

	/* Insert parameter length */
	frame[n++] = len_param;
	
	/* Insert parameter */
	memcpy( frame+n, param, len_param); n += len_param;
	
	/* Insert value length ( 2 byte little endian format, LSB first) */
	frame[n++] = len_value & 0xff;
	frame[n++] = len_value >> 8;

	/* Insert value */
	switch (type) {
	case 1:
		memcpy( frame+n, value_char, len_value); n+=len_value;
		break;
	case 2:
		frame[n++] = value_byte;
		break;
	case 3:
		frame[n++] = value_short & 0xff;
		frame[n++] = (value_short >> 8) & 0xff;
		break;
	default:
		break;
	}
	ASSERT( n == (len_param+len_value+3), return 0;);

	return len_param+len_value+3;
}

/*
 * Function irlan_get_response_param (buf, param, value)
 *
 *    Extracts a single parameter name/value pair from buffer and updates
 *    the buffer pointer to point to the next name/value pair.
 * 
 */
int irlan_get_response_param( __u8 *buf, char *name, char *value, int *len)
{
	__u8 name_len;
	__u16 val_len;
	int n=0;

	DEBUG( 4, "irlan_get_response_param()\n");
	
	/* get length of parameter name ( 1 byte) */
	name_len = buf[n++];
	
	if (name_len > 254) {
		DEBUG( 0, "extract_params, name_len > 254\n");
		return -1;
	}
	
	/* get parameter name */
	memcpy( name, buf+n, name_len);
	name[ name_len] = '\0';
	n+=name_len;
	
	/*  
	 *  Get length of parameter value ( 2 bytes in little endian 
	 *  format) 
	 */
	val_len = buf[n++] & 0xff;
	val_len |= buf[n++] << 8;
	
	if (val_len > 1016) {
		DEBUG( 0, "irlan_get_response_param()"
		       ", parameter length to long\n");
		return -1;
	}
	
	*len = val_len;

	/* get parameter value */
	memcpy( value, buf+n, val_len);
	value[ val_len] = '\0';
	n+=val_len;
	
	DEBUG( 4, "Parameter: %s ", name); 
	DEBUG( 4, "Value: %s\n", value); 

	return n;
}

/*
 * Function print_ret_code (code)
 *
 *    Print return code of request to peer IrLAN layer.
 *
 */
void print_ret_code( __u8 code) 
{
	switch( code) {
	case 0:
		printk( KERN_INFO "Success\n");
		break;
	case 1:
		printk( KERN_WARNING "Insufficient resources\n");
		break;
	case 2:
		printk( KERN_WARNING "Invalid command format\n");
		break;
	case 3:
		printk( KERN_WARNING "Command not supported\n");
		break;
	case 4:
		printk( KERN_WARNING "Parameter not supported\n");
		break;
	case 5:
		printk( KERN_WARNING "Value not supported\n");
		break;
	case 6:
		printk( KERN_WARNING "Not open\n");
		break;
	case 7:
		printk( KERN_WARNING "Authentication required\n");
		break;
	case 8:
		printk( KERN_WARNING "Invalid password\n");
		break;
	case 9:
		printk( KERN_WARNING "Protocol error\n");
		break;
	case 255:
		printk( KERN_WARNING "Asyncronous status\n");
		break;
	}
}
