/*********************************************************************
 *                
 * Filename:      irlan_common.h
 * Version:       0.1
 * Description:   IrDA LAN access layer
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sun Aug 31 20:14:37 1997
 * Modified at:   Wed Sep 23 01:07:36 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998 Dag Brattli <dagb@cs.uit.no>, All Rights Reserved.
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#ifndef IRLAN_COMMON_H
#define IRLAN_COMMON_H

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>

/* Command packet types */
#define GET_PROVIDER_INFO   0
#define GET_MEDIA_CHAR      1
#define OPEN_DATA_CHANNEL   2
#define CLOSE_DATA_CHAN     3
#define RECONNECT_DATA_CHAN 4
#define FILTER_OPERATION    5

/* Media types */
#define MEDIA_802_3 1
#define MEDIA_802_5 2

/* Filter parameters */
#define DATA_CHAN 1
#define FILTER_TYPE 2
#define FILTER_MODE 3

/* Filter types */
#define IR_DIRECTED   1
#define IR_FUNCTIONAL 2
#define IR_GROUP      3
#define IR_MAC_FRAME  4
#define IR_MULTICAST  5
#define IR_BROADCAST  6
#define IR_IPX_SOCKET 7

/* Filter modes */
#define ALL    1
#define FILTER 2
#define NONE   3

/* Filter operations */
#define GET     1
#define CLEAR   2
#define ADD     3
#define REMOVE  4
#define DYNAMIC 5

/* Access types */
#define DIRECT 1
#define PEER   2
#define HOSTED 3

#define IRLAN_MAX_HEADER (TTP_HEADER+LMP_HEADER+LAP_HEADER)

void set_multicast_list( struct device *dev);

int insert_param( struct sk_buff *skb, char *param, int type, char *value_char,
		  __u8 value_byte, __u16 value_short);
int insert_array( struct sk_buff *skb, __u8 *param, __u8 *value, 
		  __u16 len_value);
int irlan_get_response_param( __u8 *buf, char *name, char *value, int *len);
void print_ret_code( __u8 code);

#endif


