/*********************************************************************
 *
 * Filename:      irlpt.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Sun Mar  8 23:44:19 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:	  irlan.c
 *
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>,
 *			   Dag Brattli,  <dagb@cs.uit.no>
 *     All Rights Reserved.
 *
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software.
 *     This material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include "ircompat.h"
#include "irlap.h"
#include "irttp.h"
#include "irlmp.h"
#include "iriap.h"
#include "common.h"
#include "client.h"
#include "client_fsm.h"
#include "timer.h"
#include "irda.h"

/* #include "linux/config.h" */
#include "linux/module.h"

/* make depend does not understand LinuxVersionCode(2,1,0) */
#if LINUX_VERSION_CODE > 0x020100 
#include <asm/uaccess.h>
#endif
#include <linux/miscdevice.h>
#include <linux/proc_fs.h>

#if 0
static char *rcsid = "$Id: client.c,v 1.1.1.1 1998/09/29 02:27:14 root Exp $";
#endif
static char *version = "IrLPT, $Revision: 1.1.1.1 $/$Date: 1998/09/29 02:27:14 $ (Thomas Davis)";

struct file_operations client_fops = {
	seek,	/* seek */
	NULL,   /* read_irlpt (server) */
	write,
	NULL,	/* readdir */
	NULL,
	NULL,	/* ioctl */
	NULL,	/* mmap */
	open,
	close,
	NULL,
	NULL,
};

extern char *irlptstate[];

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS

/*
 * Function client_proc_read (buf, start, offset, len, unused)
 *
 */
static int client_proc_read( char *buf, char **start, off_t offset, int len, 
			     int unused)
{
	struct irlpt_cb *self;
	int index;

	len = sprintf(buf, "%s\n\n", version);
	len += sprintf(buf+len, "ifname\tminor\tservicetype\tporttype\tdaddr\tstate\tretries\tpeersap\tcount\tbuffers\n");
	len += sprintf(buf+len, "----------------------------------------------------------------------------------------------\n");

	self = (struct irlpt_cb *) hashbin_get_first( irlpt);
	while( self) {
		/* len += sprintf(buf+len, "%d\t", i); */
		len += sprintf(buf+len, "%s\t", self->ifname);
		len += sprintf(buf+len, "%d\t", self->ir_dev.minor);

		switch ( self->servicetype) {
		case UNKNOWN:
			index = 0;
			break;
		case THREE_WIRE_RAW:
			index = 1;
			break;
		case THREE_WIRE:
			index = 2;
			break;
		case NINE_WIRE:
			index = 3;
			break;
		case CENTRONICS:
			index = 4;
			break;
		case SERVER_MODE:
			index = 5;
			break;
		default:
			index = 0;
			break;
		}
		
		len += sprintf(buf+len, "%s\t", service_type[index]);
		len += sprintf(buf+len, "%s\t\t", port_type[ self->porttype]);
		len += sprintf(buf+len, "%d\t", self->daddr);
		len += sprintf(buf+len, "%s\t", fsm_state[self->state]);
		len += sprintf(buf+len, "%d\t", self->open_retries);
		len += sprintf(buf+len, "%d\t", self->dlsap_sel);

		len += sprintf(buf+len, "%d\t", self->count);
		len += sprintf(buf+len, "%d", skb_queue_len(&self->buffer));
		len += sprintf(buf+len, "\n");

		self = (struct irlpt_cb *) hashbin_get_next( irlpt);
	}

	return len;
}

struct proc_dir_entry proc_lpt_client = {
	0, 3, "lpt_client",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&client_proc_read /* get_info */,
};

extern struct proc_dir_entry proc_irda;

#endif /* PROC_FS */
#endif /* CONFIG_PROC_FS */

/*
 * Function irlpt_init (dev)
 *
 *   Initializes the irlpt control structure
 *
 */

/*int client_init( struct device *dev) {*/
int client_init(void) 
{
	DEBUG( 4, "--> client_init\n");

	printk( KERN_INFO "%s\n", version);

	irlpt = hashbin_new( HB_LOCAL); 
	if ( irlpt == NULL) {
		printk( KERN_WARNING "IrLPT: Can't allocate hashbin!\n");
		return -ENOMEM;
	}

	irlmp_register( S_PRINTER, CLIENT, client_discovery_indication);

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	proc_register_dynamic( &proc_irda, &proc_lpt_client);
#else
	proc_register( &proc_irda, &proc_lpt_client);
#endif
#endif /* CONFIG_PROC_FS */
#endif /* USE_PROC */

	DEBUG( 4, "client_init -->\n");

	return 0;
}

/*
 * Function irlpt_cleanup (void)
 *
 *
 *
 */
void client_cleanup(void)
{
	DEBUG( 4, "--> client_cleanup\n");

	irlmp_unregister( S_PRINTER, CLIENT);

	/*
	 *  Delete hashbin and close all irlan client instances in it
	 */
	hashbin_delete( irlpt, (FREE_FUNC) client_close);

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
	proc_unregister( &proc_irda, proc_lpt_client.low_ino);
#endif
#endif

	DEBUG( 4, "irlpt_cleanup -->\n");
}

/*
 * Function irlpt_open (void)
 *
 *    This is the entry-point which starts all the fun! Currently this
 *
 */
struct irlpt_cb *client_open( __u32 daddr)
{
	struct irlpt_cb *self;

	DEBUG( 3, "client_open()\n");

	self = kmalloc(sizeof(struct irlpt_cb), GFP_ATOMIC);
	if (self == NULL)
		return NULL;

	memset(self, 0, sizeof(struct irlpt_cb));

	ASSERT( self != NULL, return NULL;);

/* 	sprintf(self->ifname, "irlpt%d", handle); */
	self->ir_dev.minor = MISC_DYNAMIC_MINOR;
	self->ir_dev.name = self->ifname;
	self->ir_dev.fops = &client_fops;

	misc_register(&self->ir_dev);

	self->magic = LPT_MAGIC;
	self->in_use = TRUE;
	self->servicetype = THREE_WIRE_RAW;
	self->porttype = SERIAL;

	skb_queue_head_init(&self->buffer);

	client_next_state( self, LPT_IDLE);

	hashbin_insert( irlpt, (QUEUE *) self, daddr, NULL);

/*	MOD_INC_USE_COUNT; */

	return self;
}

/*
 * Function irlpt_close (self)
 *
 *    This function closes and marks the IrLPT instance as not in use.
 */
void client_close( struct irlpt_cb *self)
{
	struct sk_buff *skb;

	DEBUG( 3, "--> client_close()\n");

	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	while (( skb = skb_dequeue(&self->buffer)) != NULL) {
		DEBUG(3, "client_close: freeing SKB\n");
                IS_SKB( skb, return;);
                FREE_SKB_MAGIC( skb);
                DEV_KFREE_SKB( skb, FREE_READ);
	}

	misc_deregister(&self->ir_dev);

	self->magic = ~LPT_MAGIC;

	kfree( self);

/*	MOD_DEC_USE_COUNT; */

	DEBUG( 3, "client_close() -->\n");
}

/*
 * Function irlpt_discovery_indication (daddr)
 *
 *    Remote device discovered, try query the remote IAS to see which
 *    device it is, and which services it has.
 *
 */
void client_discovery_indication( __u32 daddr) {
	struct irlpt_info info;
	struct irlpt_cb *self;

	DEBUG( 0, "irlpt_discovery_indication()\n");

	ASSERT( irlpt != NULL, return ;);

	/*
	 *  Check if an instance is already dealing with this device
	 *  (daddr)
	 */
	self = (struct irlpt_cb *) hashbin_find( irlpt, daddr, NULL);
      	if ( self != NULL) {
		ASSERT( self->magic == LPT_MAGIC, return;);
		if ( self->state == LPT_IDLE) {
			client_do_event( self, IRLPT_DISCOVERY_INDICATION, NULL, &info);
		}
		return;
	}
    

	/*
	 * We have no instance for daddr, so time to start a new instance.
	 * First we must find a free entry in master array
	 */
	if (( self = client_open( daddr)) == NULL) {
		DEBUG(0, "client_open failed!\n");
	}

	ASSERT(self != NULL, return;);
	ASSERT(self->magic == LPT_MAGIC, return;);

	self->daddr = info.daddr = daddr;
	
	if (self->state == LPT_IDLE) {
		client_do_event( self, IRLPT_DISCOVERY_INDICATION, NULL, &info);
	}
}

/*
 * Function irlpt_disconnect_indication (handle)
 *
 */
void client_disconnect_indication( void *instance, void *sap, LM_REASON reason,
				   struct sk_buff *userdata)
{
	struct irlpt_info info;
	struct irlpt_cb *self;

	DEBUG( 0, "client_disconnect_indication!\n");

	self = ( struct irlpt_cb *) instance;

	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	info.daddr = self->daddr;

	DEBUG( 3, "client_disconnect_indication, reason=%d (%s), peersap=%d\n",
	       reason, reasons[reason], self->dlsap_sel);

	self->connected = DISCONNECTED;
	self->eof = reason;

	client_do_event( self, LMP_DISCONNECT, NULL, NULL);

}

/*
 * Function irlpt_connect_confirm (handle, qos, skb)
 *
 *    LSAP connection confirmed!
 */
void client_connect_confirm( void *instance, void *sap, struct qos_info *qos,
			     struct sk_buff *skb)
{
	struct irlpt_info info;
	struct irlpt_cb *self;
	
	self = ( struct irlpt_cb *) instance;
	
	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	info.daddr = self->daddr;

	DEBUG( 3, "client_connect_confirmation\n");
	
	/*
	 *  Check if we have got some QoS parameters back! This should be the
	 *  negotiated QoS for the link.
	 */
	if ( qos) {
		DEBUG( 0, "IrLPT: Negotiated BAUD_RATE: %d bps.\n", 
		       qos->baud_rate.value);
		DEBUG( 0, "IrLPT: Frame Size: %d\n",
		       qos->data_size.value);
	}

	self->connected = TRUE;
	
	client_do_event( self, LMP_CONNECT, NULL, NULL);

}

/*
 * Function client_connect_indication (handle)
 *
 */
void client_connect_indication( void *instance, void *sap, 
				struct qos_info *qos, struct sk_buff *skb)
{
	struct irlpt_cb *self;
	struct lsap_cb *lsap;
	struct irlpt_info info;

	self = ( struct irlpt_cb *) instance;
	lsap = ( struct lsap_cb *) sap;

	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	DEBUG( 3, "client_connect_indication()\n");

	self->connected = CONNECTED;
	self->eof = FALSE;

	info.lsap = lsap;

	client_do_event( self, LMP_CONNECT, NULL, &info);
}

/*
 * Function client_data_indication (handle, skb)
 *
 *    This function gets the data that is received on the data channel
 *
 */
void client_data_indication( void *instance, void *sap, struct sk_buff *skb) 
{
	struct irlpt_cb *self;

	ASSERT( skb != NULL, return;);
	DEBUG( 3, "client_data_indication: len=%d\n", (int) skb->len);

	self = ( struct irlpt_cb *) instance;

	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);
#if 0
	{
		int i;

		for(i=0;i<skb->len;i++)
			if (skb->data[i] > 31 && skb->data[i] < 128) {
				printk("%c", skb->data[i]);
			} else {
				if (skb->data[i] == 0x0d) {
					printk("\n");
				} else {
					printk(".");
				}
			}

		printk("\n");
	}
#endif
	
#if 0
	skb_queue_tail(&self->buffer, skb);
        wake_up_interruptible(&lpt_wait);
#endif
}

/*
 * Function irlpt_get_value_confirm (obj_id, type, value_int, value_char, priv)
 *
 *    Fixed to match changes in iriap.h, DB.
 *
 */
void client_get_value_confirm(__u16 obj_id, __u8 type, int value_int, 
			      char *value_char, void *priv)
{
	struct irlpt_info info;
	struct irlpt_cb *self;

	DEBUG( 3, "client_get_value_confirm()\n");

	self = (struct irlpt_cb *) priv;

	info.dlsap_sel = value_int;
	self->dlsap_sel = value_int;

	DEBUG( 3, "client_get_value_confirm: obj_id = %d, value = %d\n", 
	       obj_id, value_int);

	ASSERT( type == INTEGER, return;);

	if ( value_int != -1)
		client_do_event( self, IAS_PROVIDER_AVAIL, NULL, &info);
	else
		client_do_event( self, IAS_PROVIDER_NOT_AVAIL, NULL, &info);
}

void client_connect_request( struct irlpt_cb *self) {
	struct notify_t lpt_notify;

	DEBUG( 3, "client_connect_request()\n");

	lpt_notify.connect_confirm = client_connect_confirm;
	lpt_notify.disconnect_indication = client_disconnect_indication;
	lpt_notify.data_indication = client_data_indication;

	self->lsap = irlmp_open_lsap( LSAP_ANY, &lpt_notify);
	DEBUG( 3, "client_connect_request: Dest LSAP sel= %d\n", 
	       self->dlsap_sel);
	
	if (self->servicetype == THREE_WIRE_RAW) {
		DEBUG( 3, "client_connect_request: issue THREE_WIRE_RAW connect\n");
		irlmp_connect_request( self->lsap, self->dlsap_sel, 
				       self->daddr, NULL, NULL);
	}
}

#ifdef MODULE

#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
MODULE_AUTHOR("Thomas Davis <ratbert@radiks.net>");
MODULE_DESCRIPTION("The Linux IrDA/IrLPT protocol");
#endif

/*
 * Function init_module (void)
 *
 *    Initialize the IrLPT client module, this function is called by the
 *    modprobe(1) program.
 */
int init_module(void)
{

        DEBUG( 4, "--> irlpt client: init_module\n");

        client_init();

        DEBUG( 4, "irlpt client: init_module -->\n");

        return 0;
}

/*
 * Function cleanup_module (void)
 *
 *    Remove the IrLPT server module, this function is called by the rmmod(1)
 *    program
 */
void cleanup_module(void)
{
        DEBUG( 4, "--> irlpt client: cleanup_module\n");
        /* No need to check MOD_IN_USE, as sys_delete_module() checks. */

        /* Free some memory */
        client_cleanup();

        DEBUG( 4, "irlpt client: cleanup_module -->\n");
}

#endif /* MODULE */
