/*********************************************************************
 *
 * Filename:      common.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Sun Mar  8 23:44:19 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:	  irlpt.c
 *
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>,
 *			   Dag Brattli,  <dagb@cs.uit.no>
 *     All Rights Reserved.
 *
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software.
 *     This material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include "ircompat.h"
#include "irlap.h"
#include "irttp.h"
#include "irlmp.h"
#include "iriap.h"
#include "timer.h"
#include "irda.h"

#include "common.h"
#include "client.h"
#include "server.h"

#include <asm/segment.h>

/* make depend does not understand LinuxVersionCode(2,1,0) */
#if LINUX_VERSION_CODE > 0x020100 
#include <asm/uaccess.h>
#endif
#include <linux/miscdevice.h>
#include <linux/proc_fs.h>

char *service_type[] = {
	"Unknown",
	"THREE_WIRE_RAW",
	"THREE_WIRE",
	"NINE_WIRE",
	"CENTRONICS",
	"SERVER_MODE",
};

char *port_type[] = {
	"Unknown",
	"SERIAL",
	"PARALLEL",
};

char *connected[] = {
	"DISCONNECTED",
	"WAITING",
	"CONNECTED",
	"FLUSHED",
};

char *reasons[] = {
	"SERVICE_CLOSE",     /* Service has closed the connection */
	"DISC_INDICATION",   /* Received a disconnect request from peer entity*/
	"NO_RESPONSE",       /* To many retransmits without response */
	"DEADLOCK_DETECTED", /* To many retransmits _with_ response */
	"FOUND_NONE",        /* No devices were discovered */
	"MEDIA_BUSY",

};

char *fsm_state[] = {
	"IDLE",
	"QUERY",
	"READY",
	"WAITI",
	"WAITR",
	"CONN"
};

char *fsm_event[] = {
        "IRLPT_DISCOVERY_INDICATION",
	"QUERY_REMOTE_IAS",
	"IAS_PROVIDER_AVAIL",
	"IAS_PROVIDER_NOT_AVAIL",
	"LAP_DISCONNECT",
	"LMP_CONNECT",
	"LMP_DISCONNECT",
	"LMP_CONNECT_INDICATION",
	"LMP_DISCONNECT_INDICATION",
	"LPT_CONNECT_REQUEST",
	"LPT_DISCONNECT_REQUEST",
	"CLIENT_DATA_INDICATION",
};

HASHBIN *irlpt;
struct irlpt_cb *irlptd = NULL;

static struct wait_queue *lpt_wait;

#if 0
static char *rcsid = "$Id: common.c,v 1.2 1998/10/13 22:21:17 ryan Exp $";
#endif

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
struct irlpt_cb *find_handle(struct inode *inode)
{
	unsigned int minor = MINOR( inode->i_rdev);
#else
struct irlpt_cb *find_handle(struct file *file)
{
	unsigned int minor = MINOR( file->f_dentry->d_inode->i_rdev);
#endif
	struct irlpt_cb *self;

	DEBUG(4, "find_handle:\n");
	
	/* Check for server */
	if (irlptd != NULL && irlptd->ir_dev.minor == minor) {
		DEBUG( 4, "find_handle: irlptd file handle!\n");
		return irlptd;
	}

	/* Check the clients */
	self = (struct irlpt_cb *) hashbin_get_first( irlpt);
	while ( self) {
		ASSERT( self->magic == LPT_MAGIC, return NULL;);
		
		if ( minor == self->ir_dev.minor)
			return self;

		self = (struct irlpt_cb *) hashbin_get_next( irlpt);
	}
	return NULL;
}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
int read( struct inode * inode, struct file * file, char * buffer,
	  int count)
#else
ssize_t read( struct file *file, char *buffer, size_t count, loff_t
	      *noidea)
#endif
{
	int len=0;
	char *ptr = buffer;
	struct irlpt_cb *self;
	struct sk_buff *skb = NULL;
	struct wait_queue wait = { current, NULL };

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

	ASSERT( self != NULL, return 0;);
	ASSERT( self->magic == LPT_MAGIC, return 0;);

	DEBUG( 3, "read_irlpt: count=%d, skb_len=%d, connected=%d (%s) eof=%d\n", 
	       count, skb_queue_len(&self->buffer), self->connected, 
	       connected[self->connected], self->eof);

	if (self->eof && !skb_queue_len(&self->buffer)) {
		switch (self->eof) {
		case LM_USER_REQUEST:
			irlptd->eof = FALSE;
			DEBUG(3, "read_irlpt: returning 0\n");
			return 0;
		case LM_LAP_DISCONNECT:
			irlptd->eof = FALSE;
			return -EIO;
		case LM_LAP_RESET:
			irlptd->eof = FALSE;
			return -ECONNRESET;
		default:
			irlptd->eof = FALSE;
			return -EIO;
		}
	}

	add_wait_queue(&lpt_wait, &wait);

	while (len <= count) {
		skb = skb_dequeue(&self->buffer);

		if (skb != NULL) {
			DEBUG( 3, "read_irlpt: len=%d, skb->len=%d, count=%d\n", 
			       len, (int) skb->len, count);

			if ((skb->len + len) < count) {
				copy_to_user(ptr, skb->data, skb->len);
				len += skb->len;
				ptr += skb->len;
		
				IS_SKB( skb, return -1;);
				FREE_SKB_MAGIC( skb);
				DEV_KFREE_SKB( skb, FREE_READ);
			} else {
				skb_queue_head(&self->buffer, skb);
				break;
			}
		} else {
			DEBUG( 3, "read_irlpt: skb=NULL, len=%d, count=%d, eof=%d\n", 
			       len, count, self->eof);
#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
			if (!signal_pending(current) && !self->eof) {
#else
			if (!current->signal && ~current->blocked && 
			    !self->eof) {
#endif
				current->state = TASK_INTERRUPTIBLE;
				schedule();
			} else
				break;
		}
	}

	current->state = TASK_RUNNING;
	remove_wait_queue(&lpt_wait, &wait);

	DEBUG( 3, "read_irlpt: len=%d\n", len);
	return len;
}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
int write(struct inode * inode, struct file * file,
		 const char *buffer, int count)
#else
ssize_t write(struct file *file, const char *buffer,
		     size_t count, loff_t *noidea)
#endif
{
	struct irlpt_cb *self;
	struct sk_buff *skb;

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

	ASSERT( self != NULL, return 0;);
	ASSERT( self->magic == LPT_MAGIC, return 0;);
	
	DEBUG( 4, "write: count = %d\n", count);
	
	if (self->state != LPT_CONN) {
		DEBUG( 3, "write: state != LPT_CONN (possible link problems?)\n");
		return -ENOLINK;
	}

	count -= 4;
	/* FIXME! */
/* 	if (count > (irlap->qos_session.data_size_value - LMP_HEADER - LAP_HEADER)) { */
/* 		count = irlap->qos_session.data_size_value - LMP_HEADER - LAP_HEADER; */
/* 		DEBUG(4, "write: setting count to %d\n", count); */
/* 	} */

	skb = dev_alloc_skb(count + IRLPT_MAX_HEADER);
	if ( skb == NULL) {
		DEBUG(0, "irlpt - couldn't allocate skbuff!\n");
		return 0;
	}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	skb->free = 1;
#endif
	skb_reserve( skb, IRLPT_MAX_HEADER);
	skb_put( skb, count);

	copy_from_user( skb->data, buffer, count);

	irlmp_data_request(self->lsap, skb);

	return(count);
}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
int seek( struct inode * inode, struct file * file,
	  off_t offset, int count)
#else
loff_t seek( struct file *file, loff_t offset, int count)
#endif
{
	return -ESPIPE;
}

/*
 * Function open_irlpt (inode, file)
 *
 *
 *
 */
int open(struct inode * inode, struct file *file)
{
	struct irlpt_cb *self;

	DEBUG( 4, "open:\n");

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

#if 0
	if (self->state == LPT_IDLE) {
		DEBUG( 3, "open_irlpt: state == LPT_IDLE! (no device found yet)\n");
		return -ENODEV;
	}

	if (self->state == LPT_QUERY ||
	    self->state == LPT_READY ||
	    self->state == LPT_WAITI) {
		DEBUG( 3, "open_irlpt: state == LPT_QUERY, LPT_READY or LPT_WAITI (link problems)!\n");
		return -EBUSY;
	}
#endif

	if (self->count++) {
		DEBUG( 3, "open_irlpt: count not zero; actual = %d\n",
		       self->count);
		self->count--;
		return -EBUSY;
	}

	return 0;
}

/*
 * Function close_irlpt (inode, file)
 *
 *
 *
 */
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
void close(struct inode *inode, struct file *file)
#else
int  close(struct inode *inode, struct file *file)
#endif
{
	struct irlpt_cb *self;

	DEBUG( 4, "close_irlpt()\n");
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

	self->count--;

#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
	return 0;
#endif	
}

void dump_buffer( struct sk_buff *skb) 
{
	int i;

	for(i=0;i<skb->len;i++)
		if (skb->data[i] > 31 && skb->data[i] < 128) {
			printk("%c", skb->data[i]);
		} else {
			if (skb->data[i] == 0x0d) {
				printk("\n");
			} else {
				printk(".");
			}
		}

	printk("\n");
}
