/*********************************************************************
 *
 * Filename:      irlpt.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Sun Mar  8 23:44:19 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:	  irlan.c
 *
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>,
 *			   Dag Brattli,  <dagb@cs.uit.no>
 *     All Rights Reserved.
 *
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software.
 *     This material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#ifndef IRLPT_COMMON_H
#define IRLPT_COMMON_H

#include "qos.h"
#include "irmod.h"

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/miscdevice.h>

extern char *service_type[];
extern char *port_type[];
extern char *connected[];
extern char *reasons[];
extern char *fsm_state[];
extern char *fsm_event[];

extern struct wait_queue *lpt_wait;

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
extern struct irlpt_cb *find_handle(struct inode *inode);
extern int read(struct inode * inode, struct file * file, char * buffer, int count);
extern int write(struct inode * inode, struct file * file,const char *buffer, int count);
extern int seek(struct inode * inode, struct file * file, off_t offset,
		int count);
extern int open(struct inode *indoe, struct file *file);
extern void close(struct inode *inode, struct file *file);
#else
extern struct irlpt_cb *find_handle(struct file *file);
extern ssize_t read( struct file *file, char *buffer, size_t count, 
		     loff_t *noidea);
extern ssize_t write(struct file *file, const char *buffer, size_t count, 
		     loff_t *noidea);
extern loff_t seek(struct file *, loff_t, int);
extern int open(struct inode * inode, struct file *file);
extern int close(struct inode *inode, struct file *file);
#endif

/* FSM definitions */

typedef enum {
        LPT_IDLE,
	LPT_QUERY,
	LPT_READY,
	LPT_WAITI,
	LPT_WAITR,
	LPT_CONN,
} IRLPT_STATE;

/* IrLPT Events */
typedef enum {
        IRLPT_DISCOVERY_INDICATION,
	QUERY_REMOTE_IAS,
	IAS_PROVIDER_AVAIL,
	IAS_PROVIDER_NOT_AVAIL,
	LAP_DISCONNECT,
	LMP_CONNECT,
	LMP_DISCONNECT,
	LMP_CONNECT_INDICATION,
	LMP_DISCONNECT_INDICATION,
#if 0
	TTP_CONNECT_INDICATION,
	TTP_DISCONNECT_INDICATION,
#endif
	LPT_CONNECT_REQUEST,
	LPT_DISCONNECT_REQUEST,
	CLIENT_DATA_INDICATION,
} IRLPT_EVENT;

struct irlpt_info {
	struct lsap_cb *lsap;
	__u8 dlsap_sel;
	__u32 daddr;
};

/* Command packet types */

#define IRLPT_MAX_PACKET 1024
#define IRLPT_MAX_HEADER LMP_HEADER+LAP_HEADER
#define LPT_MAX_DEVICES 3
#define LPT_MAGIC 0x0755

typedef enum {
	DISCONNECTED,
	WAITING,
	CONNECTED,
	FLUSHED,
} IRLPT_SERVER_STATE;

#define PRINTER_LSAP    0x09

#define PI_SERVICE_TYPE	0x00

#define UNKNOWN         0x00            /* not defined yet. */
#define THREE_WIRE_RAW	0x01		/* bit 0 */
#define THREE_WIRE	0x02		/* bit 1 */
#define NINE_WIRE	0x04		/* bit 2 */
#define CENTRONICS	0x08		/* bit 3 */
#define SERVER_MODE     0xFF            /* our own flag */

#define PI_PORT_TYPE	0x01

#define SERIAL		0x01		/* bit 0 */
#define PARALLEL	0x02		/* bit 1 */

#define PI_PORT_NAME	0x02

#define PI_CRITICAL	0x80


struct irlpt_cb {
        int magic;          /* magic used to detect corruption of the struct */
	__u32 daddr;         /* my local address. */

	struct timer_list retry_timer;
	
	int volatile state; /* Current state of IrCOMM layer */
	int open_retries;
        int in_use;         /* flag to prevent re-use */
        char ifname[9];     /* name of the allocated instance, and registered device. */
	struct lsap_cb *lsap;         /* lmp handle */

	__u8 dlsap_sel;     /* remote LSAP selector address */
	__u8 slsap_sel;     /* local LSAP selectoraddress */
	__u8 servicetype;   /* Type of remote service, ie THREE_WIRE_RAW */
	__u8 porttype;      /* type of remote port. */

	struct miscdevice ir_dev; /* used to register the misc device. */

	int count;                /* open count */

	/* this is used by the server side of the system */

        IRLPT_SERVER_STATE connected;
	int eof;
	int service_LSAP;

	struct sk_buff_head buffer; /* buffer queue */

	/*  Used by extract_params, placed here for now to avoid placing
	 *  them on the stack 
	 */
	char name[254];
	char value[1016];
};

/* Debug function */

/* #define CHECK_SKB(skb) check_skb((skb),  __LINE__,__FILE__) */

void dump_buffer(struct sk_buff *);

#endif




