/*********************************************************************
 *
 * Filename:      irlpt.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Sun Mar  8 23:44:19 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:	  irlan.c
 *
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>,
 *			   Dag Brattli,  <dagb@cs.uit.no>
 *     All Rights Reserved.
 *
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software.
 *     This material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include "ircompat.h"
#include "irlap.h"
#include "irttp.h"
#include "irlmp.h"
#include "iriap.h"
#include "irlpt.h"
#include "irlpt_event.h"
#include "timer.h"
#include "irda.h"

/* #include "linux/config.h" */
/* #include "linux/module.h" */

/* make depend does not understand LinuxVersionCode(2,1,0) */
#if LINUX_VERSION_CODE > 0x020100 
#include <asm/uaccess.h>
#endif
#include <linux/miscdevice.h>
#include <linux/proc_fs.h>

static char *irlptservicetype[] = {
	"Unknown",
	"THREE_WIRE_RAW",
	"THREE_WIRE",
	"NINE_WIRE",
	"CENTRONICS",
	"SERVER_MODE",
};

static char *irlptporttype[] = {
	"Unknown",
	"SERIAL",
	"PARALLEL",
};

static char *connected[] = {
	"DISCONNECTED",
	"WAITING",
	"CONNECTED",
	"FLUSHED",
};

struct irlpt_cb **irlpt = NULL;
struct irlpt_cb *irlptd = NULL;

static struct wait_queue *lpt_wait;

#if 0
static char *rcsid = "$Id: irlpt.c,v 1.1.1.1 1998/09/29 02:27:14 root Exp $";
#endif
static char *version = "IrLPT, $Revision: 1.1.1.1 $/$Date: 1998/09/29 02:27:14 $ (Thomas Davis)";

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static struct irlpt_cb *find_handle(struct inode *inode)
{
	unsigned int minor = MINOR( inode->i_rdev);
#else
static struct irlpt_cb *find_handle(struct file *file)
{
	unsigned int minor = MINOR( file->f_dentry->d_inode->i_rdev);
#endif
	int i;

	DEBUG(4, "find_handle:\n");

	if (irlptd->ir_dev.minor == minor) {
		DEBUG( 4, "find_handle: irlptd file handle!\n");
		return irlptd;
	}

	for (i=0; i<LPT_MAX_DEVICES; i++) {
		if (irlpt[i]) {
			if (irlpt[i]->ir_dev.minor == minor) {
				DEBUG(4, "find_handle: found device %s, minor = %d, handle = %d\n", 
				      irlpt[i]->ifname, irlpt[i]->ir_dev.minor, i);
				return irlpt[i];
			}
		}
	}

	return NULL;

}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static int read_irlpt(struct inode * inode, struct file * file, char * buffer,
		      int count)
#else
ssize_t read_irlpt( struct file *file, char *buffer, size_t count, loff_t
		    *noidea)
#endif
{
	int len=0;
	char *ptr = buffer;
	struct irlpt_cb *self;
	struct sk_buff *skb = NULL;
	struct wait_queue wait = { current, NULL };

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

	DEBUG( 3, "read_irlpt: count=%d, skb_len=%d, connected=%d (%s) eof=%d\n", 
	       count, skb_queue_len(&self->buffer), self->connected, 
	       connected[self->connected], self->eof);

	if (self->eof && !skb_queue_len(&self->buffer)) {
		switch (self->eof) {
		case SERVICE_CLOSE:
			irlptd->eof = FALSE;
			DEBUG(3, "read_irlpt: returning 0\n");
			return 0;
		case DISC_INDICATION:
			irlptd->eof = FALSE;
			return -EIO;
		case RESET_INDICATION:
			irlptd->eof = FALSE;
			return -ECONNRESET;
		default:
			irlptd->eof = FALSE;
			return -EIO;
		}
	}

	add_wait_queue(&lpt_wait, &wait);

	while (len <= count) {
		skb = skb_dequeue(&self->buffer);

		if (skb != NULL) {
			DEBUG( 3, "read_irlpt: len=%d, skb->len=%d, count=%d\n", 
			       len, (int) skb->len, count);

			if ((skb->len + len) < count) {
				copy_to_user(ptr, skb->data, skb->len);
				len += skb->len;
				ptr += skb->len;
		
				IS_SKB( skb, return;);
				FREE_SKB_MAGIC( skb);
				DEV_KFREE_SKB( skb, FREE_READ);
			} else {
				skb_queue_head(&self->buffer, skb);
				break;
			}
		} else {
			DEBUG( 3, "read_irlpt: skb=NULL, len=%d, count=%d, eof=%d\n", 
			       len, count, self->eof);
#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
			if (!signal_pending(current) && !self->eof) {
#else
			if (!current->signal && ~current->blocked && 
			    !self->eof) {
#endif
				current->state = TASK_INTERRUPTIBLE;
				schedule();
			} else
				break;
		}
	}

	current->state = TASK_RUNNING;
	remove_wait_queue(&lpt_wait, &wait);

	DEBUG( 3, "read_irlpt: len=%d\n", len);
	return len;
}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static int write_irlpt(struct inode * inode, struct file * file,
		       const char *buffer, int count)
#else
static ssize_t write_irlpt (struct file *file, const char *buffer,
			    size_t count, loff_t *noidea)
#endif
{
	struct irlpt_cb *self;
	struct sk_buff *skb;

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

	DEBUG( 4, "write_irlpt: count = %d\n", count);

	if (self->state != LPT_CONN) {
		DEBUG( 3, "write_irlpt: state != LPT_CONN (possible link problems?)\n");
		return -ENOLINK;
	}

	if (count > (irlap->qos_session.data_size_value - LMP_HEADER - LAP_HEADER)) {
		count = irlap->qos_session.data_size_value - LMP_HEADER - LAP_HEADER;
		DEBUG(4, "write_irlpt: setting count to %d\n", count);
	}

	skb = dev_alloc_skb(count + IRLPT_MAX_HEADER);
	if ( skb == NULL) {
		DEBUG(0, "irlpt - couldn't allocate skbuff!\n");
		return 0;
	}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	skb->free = 1;
#endif
	skb_reserve( skb, IRLPT_MAX_HEADER);
	skb->len = count;

	copy_from_user( skb->data, buffer, count);

	irlmp_data_request(self->lsap, skb);

	return(count);
}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static int seek_irlpt(struct inode * inode, struct file * file,
		       const char *buffer, int count)
#else
static ssize_t seek_irlpt (struct file *file, const char *buffer,
			    size_t count, loff_t *noidea)
#endif
{
	return -ESPIPE;
}
/*
 * Function open_irlpt (inode, file)
 *
 *
 *
 */
static int open_irlpt(struct inode * inode, struct file *file)
{
	struct irlpt_cb *self;

	DEBUG( 4, "open_irlpt:\n");

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

#if 0
	if (self->state == LPT_IDLE) {
		DEBUG( 3, "open_irlpt: state == LPT_IDLE! (no device found yet)\n");
		return -ENODEV;
	}

	if (self->state == LPT_QUERY ||
	    self->state == LPT_READY ||
	    self->state == LPT_WAITI) {
		DEBUG( 3, "open_irlpt: state == LPT_QUERY, LPT_READY or LPT_WAITI (link problems)!\n");
		return -EBUSY;
	}
#endif

	if (self->count++) {
		DEBUG( 3, "open_irlpt: count not zero; actual = %d\n",
		       self->count);
		self->count--;
		return -EBUSY;
	}

	return 0;
}

/*
 * Function close_irlpt (inode, file)
 *
 *
 *
 */
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static void close_irlpt(struct inode *inode, struct file *file)
#else
static int  close_irlpt(struct inode *inode, struct file *file)
#endif
{
	struct irlpt_cb *self;

	DEBUG( 4, "close_irlpt()\n");
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	self = find_handle(inode);
#else
	self = find_handle(file);
#endif

	self->count--;

#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
	return 0;
#endif	
}

struct file_operations irlpt_fops = {
	seek_irlpt,		/* seek */
	read_irlpt,
	write_irlpt,
	NULL,		/* readdir */
	NULL,
	NULL,		/* ioctl */
	NULL,		/* mmap */
	open_irlpt,
	close_irlpt,
	NULL,
	NULL,
};

extern char *irlptstate[];

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS

/*
 * Function proc_irlpt_read (buf, start, offset, len, unused)
 *
 *
 *
 */
int irlpt_proc_read(char *buf, char **start, off_t offset, int len, int unused)
{
	int i, index;

	len = 0;
	len = sprintf(buf, "%s\n\n", version);
	len += sprintf(buf+len, "ifname\tminor\tservicetype\tporttype\tdaddr\tstate\tretries\tpeersap\tcount\tbuffers\n");
	len += sprintf(buf+len, "----------------------------------------------------------------------------------------------\n");
	for (i=0; i<LPT_MAX_DEVICES; i++) {
		if (irlpt[i] != NULL) {
			/* len += sprintf(buf+len, "%d\t", i); */
			len += sprintf(buf+len, "%s\t", irlpt[i]->ifname);
			len += sprintf(buf+len, "%d\t", irlpt[i]->ir_dev.minor);

			switch (irlpt[i]->servicetype) {
			case UNKNOWN:
				index = 0;
				break;
			case THREE_WIRE_RAW:
				index = 1;
				break;
			case THREE_WIRE:
				index = 2;
				break;
			case NINE_WIRE:
				index = 3;
				break;
			case CENTRONICS:
				index = 4;
				break;
			case SERVER_MODE:
				index = 5;
				break;
			default:
				index = 0;
				break;
			}

			len += sprintf(buf+len, "%s\t", irlptservicetype[index]);
			len += sprintf(buf+len, "%s\t\t", irlptporttype[irlpt[i]->porttype]);
			len += sprintf(buf+len, "%d\t", irlpt[i]->daddr);
			len += sprintf(buf+len, "%s\t", irlptstate[irlpt[i]->state]);
			len += sprintf(buf+len, "%d\t", irlpt[i]->open_retries);
			len += sprintf(buf+len, "%d\t", irlpt[i]->peersap);
			/* len += sprintf(buf+len, "data_handle: %d\n", irlpt[i]->data_handle); */
			len += sprintf(buf+len, "%d\t", irlpt[i]->count);
			len += sprintf(buf+len, "%d", skb_queue_len(&irlpt[i]->buffer));
			len += sprintf(buf+len, "\n");
		}
	}

	if (irlptd != NULL) {
		len += sprintf(buf+len, "%s\t", irlptd->ifname);
		len += sprintf(buf+len, "%d\t", irlptd->ir_dev.minor);

		switch (irlptd->servicetype) {
		case UNKNOWN:
			index = 0;
			break;
		case THREE_WIRE_RAW:
			index = 1;
			break;
		case THREE_WIRE:
			index = 2;
			break;
		case NINE_WIRE:
			index = 3;
			break;
		case CENTRONICS:
			index = 4;
			break;
		case SERVER_MODE:
			index = 5;
			break;
		default:
			index = 0;
			break;
		}

		len += sprintf(buf+len, "%s\t", irlptservicetype[index]);
		len += sprintf(buf+len, "%s\t\t", irlptporttype[irlptd->porttype]);
		len += sprintf(buf+len, "%d\t", irlptd->daddr);
		len += sprintf(buf+len, "%s\t", irlptstate[irlptd->state]);
		len += sprintf(buf+len, "%d\t", irlptd->open_retries);
		len += sprintf(buf+len, "%d\t", irlptd->peersap);
		len += sprintf(buf+len, "%d\t", irlptd->count);
		len += sprintf(buf+len, "%d", skb_queue_len(&irlptd->buffer));
		len += sprintf(buf+len, "\n");
	}
	return len;
}

extern struct proc_dir_entry proc_irda;
extern struct proc_dir_entry proc_lpt;

#endif /* PROC_FS */
#endif /* CONFIG_PROC_FS */

/*
 * Function irlpt_init (dev)
 *
 *   Initializes the irlpt control structure
 *
 */

/*int irlpt_init( struct device *dev) {*/
int irlpt_init(void) {
	DEBUG( 4, "--> irlpt_init\n");

	printk( KERN_INFO "%s\n", version);

	irlpt = (struct irlpt_cb **) kmalloc( sizeof(void *) * LPT_MAX_DEVICES,
					      GFP_KERNEL);

	if ( irlpt == NULL) {
		printk( KERN_WARNING "IrLPT: Can't allocate irlpt array!\n");
		return -ENOMEM;
	}

	memset( irlpt, 0, sizeof(void *) * LPT_MAX_DEVICES);

	irlmp_register_client( PRINTER, irlpt_discovery_indication);

	irlptd = (struct irlpt_cb *) kmalloc (sizeof(struct irlpt_cb), GFP_KERNEL);

	if ( irlptd == NULL) {
		printk( KERN_WARNING "IrLPT: Can't allocate irlptd control block!\n");
		return -ENOMEM;
	}

	memset( irlptd, 0, sizeof(struct irlpt_cb));

	sprintf(irlptd->ifname, "irlptd");
	irlptd->ir_dev.minor = MISC_DYNAMIC_MINOR;
	irlptd->ir_dev.name = irlptd->ifname;
	irlptd->ir_dev.fops = &irlpt_fops;
	misc_register(&irlptd->ir_dev);
	irlptd->magic = LPT_MAGIC;
	irlptd->in_use = TRUE;
	irlptd->servicetype = THREE_WIRE_RAW;
	irlptd->porttype = SERIAL;
	irlptd->lsap = NO_HANDLE;

	skb_queue_head_init(&irlptd->buffer);

	irlpt_register_server();

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	proc_register_dynamic( &proc_irda, &proc_lpt);
#else
	proc_register( &proc_irda, &proc_lpt);
#endif
#endif /* CONFIG_PROC_FS */
#endif /* USE_PROC */

	DEBUG( 4, "irlpt_init -->\n");

	return 0;
}

/*
 * Function irlpt_cleanup (void)
 *
 *
 *
 */
void irlpt_cleanup(void)
{
	int i;
	struct sk_buff *skb;

	DEBUG( 4, "--> irlpt_cleanup\n");

	if ( irlpt != NULL) {
		for (i=0; i<LPT_MAX_DEVICES; i++) {
			if (irlpt[i]) {
				if (irlpt[i]->in_use) {
					irlpt_close(irlpt[i]);
				}
				kfree(irlpt[i]);
				irlpt[i] = NULL;
			}
		}
		DEBUG( 4, "freeing master array\n");
		kfree(irlpt);
		irlpt = NULL;
	}

	while (( skb = skb_dequeue(&irlptd->buffer)) != NULL) {
		DEBUG(3, "irlpt_init: freeing SKB\n");
                IS_SKB( skb, return;);
                FREE_SKB_MAGIC( skb);
                DEV_KFREE_SKB( skb, FREE_READ);
	}

	misc_deregister(&irlptd->ir_dev);

	kfree(irlptd);

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
	proc_unregister( &proc_irda, proc_lpt.low_ino);
#endif
#endif

	DEBUG( 4, "irlpt_cleanup -->\n");
}

/*
 * Function irlpt_open (void)
 *
 *    This is the entry-point which starts all the fun! Currently this
 *    function gets called when the user issues an ifconfig command in the
 *    shell. Should been done in a better way.
 *
 */
struct irlpt_cb *irlpt_open(void)
{
	struct irlpt_cb *self;
	int handle = NO_HANDLE, i;

	DEBUG( 3, "irlpt_open()\n");

	for (i=0; i<LPT_MAX_DEVICES;i++) {
		self = irlpt[i];

		if (self == NULL) {
			handle = i;
			DEBUG( 4, "irlpt: allocated %d\n", i);
			break; /* Found free channel */
		}
		if (!self->in_use) {
			handle = i;
			DEBUG( 4, "irlpt: reusing %d\n", i);
			break; /* Found allocated but not in use */
		}
	}

	if (handle == NO_HANDLE) {
		printk( KERN_ERR "IrLPT: table is full!!\n");
		return NULL;
	}

	/* Start LPT layer only once */
	if (irlpt[handle] == NULL) {
		DEBUG( 4, "irlpt_open: allocating entry = %d\n", handle);
		self = kmalloc(sizeof(struct irlpt_cb), GFP_ATOMIC);
		if (self == NULL)
			return NULL;

		memset(self, 0, sizeof(struct irlpt_cb));
		irlpt[handle] = self;
	} else {
		DEBUG( 4, "irlpt_open: reusing entry = %d\n", handle);

		self = irlpt[handle];

		if (self->magic != LPT_MAGIC) {
			DEBUG(3, "irlpt_open: entry has bad magic!\n");
			return NULL;
		}
	}

	ASSERT( self != NULL, return NULL;);

	sprintf(self->ifname, "irlpt%d", handle);
	self->ir_dev.minor = MISC_DYNAMIC_MINOR;
	self->ir_dev.name = self->ifname;
	self->ir_dev.fops = &irlpt_fops;
	misc_register(&self->ir_dev);
	self->magic = LPT_MAGIC;
	self->in_use = TRUE;
	self->servicetype = THREE_WIRE_RAW;
	self->porttype = SERIAL;
	self->lsap = NO_HANDLE;

	skb_queue_head_init(&self->buffer);

	irlpt_next_state( self, LPT_IDLE);

	/* MOD_INC_USE_COUNT; */

	return self;
}

/*
 * Function irlpt_close (self)
 *
 *    This function closes and marks the IrLPT instance as not in use.
 */
void irlpt_close( struct irlpt_cb *self)
{
	struct sk_buff *skb;

	DEBUG( 3, "--> irlpt_close()\n");

	ASSERT( self != NULL, return;);

	/* First make sure we're connected. */
	if ( !self || self->magic != LPT_MAGIC) {
		DEBUG( 0, "irlpt_close: bad pointer *self or bad magic!\n");
		return;
	}

	while (( skb = skb_dequeue(&self->buffer)) != NULL) {
		DEBUG(3, "irlpt_close: freeing SKB\n");
                IS_SKB( skb, return;);
                FREE_SKB_MAGIC( skb);
                DEV_KFREE_SKB( skb, FREE_READ);
	}

	misc_deregister(&self->ir_dev);

	self->in_use = FALSE;

	/* Clear structure */
	memset( self, 0, sizeof(struct irlpt_cb));

	/* Set magic since the structure exists (is allocated) */
	self->magic = LPT_MAGIC;

	/* MOD_DEC_USE_COUNT; */

	DEBUG( 3, "irlpt_close() -->\n");
}

/*
 * Function irlpt_discovery_indication (daddr)
 *
 *    Remote device discovered, try query the remote IAS to see which
 *    device it is, and which services it has.
 *
 */
void irlpt_discovery_indication( __u32 daddr) {
	struct irlpt_frame info;
	struct irlpt_cb *self;
#if 0
      	int handle = NO_HANDLE;
#endif
	int i;

	DEBUG( 4, "\nirlpt_discovery_indication()\n");

	ASSERT( irlpt != NULL, return ;);

	/*
	 *  Check if an instance is allready dealing with this device
	 *  (daddr)
	 */
	for (i=0; i<LPT_MAX_DEVICES;i++) {
		/*
		 *  Check if instance exists, and is in use
		 */
		self = irlpt[i];
		if (( self != NULL) && ( self->in_use)) {
			if ( self->daddr == daddr) {
				if ( self->state == LPT_IDLE) {
					irlpt_do_event( self, IRLPT_DISCOVERY_INDICATION, NULL, &info);
				}
				DEBUG( 3, "irlpt_discovery_indication:"
				       " device already discovered\n");
				return;
			}
		}
	}

	/*
	 * We have no instance for daddr, so time to start a new instance.
	 * First we must find a free entry in master array
	 */
	if (( self = irlpt_open()) == NULL) {
		DEBUG(0, "irlpt_open failed!\n");
	}

	ASSERT(self != NULL, return;);
	ASSERT(self->magic == LPT_MAGIC, return;);

	self->daddr = info.daddr = daddr;

	if (self->state == LPT_IDLE) {
		irlpt_do_event( self, IRLPT_DISCOVERY_INDICATION, NULL, &info);
	}
}

static struct irlpt_cb *find_lmp_handle(int handle) {
	int i;

	DEBUG( 4, "find_lmp_handle:\n");

	if (irlptd->lsap == handle) {
		DEBUG( 4, "find_lmp_handle: irlptd file handle!\n");
		return irlptd;
	}

	for (i=0; i<LPT_MAX_DEVICES; i++) {
		if (irlpt[i]) {
			if (irlpt[i]->lsap == handle) {
				DEBUG( 4, "find_lmp_handle: found handle %d\n", handle);
				return irlpt[i];
			}
		}
	}
	return NULL;

}

char *reasons[] = {
	"SERVICE_CLOSE",     /* Service has closed the connection */
	"DISC_INDICATION",   /* Received a disconnect request from peer entity*/
	"NO_RESPONSE",       /* To many retransmits without response */
	"DEADLOCK_DETECTED", /* To many retransmits _with_ response */
	"FOUND_NONE",        /* No devices were discovered */
	"MEDIA_BUSY",

};

/*
 * Function irlpt_disconnect_indication (handle)
 *
 */
void irlpt_disconnect_indication( int handle, REASON reason)
{
	struct irlpt_frame info;
	struct irlpt_cb *self;
/* 	struct wait_queue wait = { current, NULL }; */

	self = find_lmp_handle(handle);

	info.daddr = self->daddr;

	DEBUG( 3, "irlpt_disconnect_indication, handle=%d, reason=%d (%s), peersap=%d\n",
	       handle, reason, reasons[reason], self->peersap);

	self->connected = DISCONNECTED;
	self->eof = reason;

        wake_up_interruptible(&lpt_wait);

	DEBUG( 3, "irlpt_disconnect_indication: skb_queue_len=%d\n",
	       skb_queue_len(&irlptd->buffer));

#if 0
	add_wait_queue(&lpt_wait, &wait);

	while (skb_queue_len(&irlptd->buffer) > 0 || self->connected != FLUSHED) {
		wake_up_interruptible(&lpt_wait);
		current->state = TASK_INTERRUPTIBLE;
		schedule();
	}

	current->state = TASK_RUNNING;
	remove_wait_queue(&lpt_wait, &wait);
#endif

	irlpt_do_event( self, LMP_DISCONNECT, NULL, NULL);

	irlpt_register_server();
}

/*
 * Function irlpt_connect_confirm (handle, qos, skb)
 *
 *    LSAP connection confirmed!
 */
void irlpt_connect_confirm( int handle, struct qos_info *qos,
			    struct sk_buff *skb)
{
	struct irlpt_frame info;
	struct irlpt_cb *self;

	self = find_lmp_handle(handle);

	info.daddr = self->daddr;

	DEBUG( 3, "irlpt_connect_confirmation\n");

	/*
	 *  Check if we have got some QoS parameters back! This should be the
	 *  negotiated QoS for the link.
	 */
	if ( qos) {
		DEBUG( 0, "IrLPT, Negotiated BAUD_RATE: %02x\n", 
		       qos->baud_rate);			
		DEBUG( 0, "IrLPT, Negotiated BAUD_RATE: %d bps.\n", 
		       qos->baud_rate_value);
	}

	self->connected = TRUE;

	irlpt_do_event( self, LMP_CONNECT, NULL, NULL);

}

/*
 * Function irlpt_connect_indication (handle)
 *
 */
void irlpt_connect_indication( int handle, int dlsap, struct sk_buff *skb)
{
	struct irlpt_cb *self;
	struct irlpt_frame info;

	DEBUG( 3, "irlpt_connect_indication, lmp_handle = %d\n", handle);

	self = find_lmp_handle(handle);

	self->handle = handle;
	self->connected = CONNECTED;
	self->eof = FALSE;

	info.dlsap = dlsap;

	irlpt_do_event( self, LMP_CONNECT, NULL, &info);
}

/*
 * Function irlpt_data_indication (handle, skb)
 *
 *    This function gets the data that is received on the data channel
 *
 */
void irlpt_data_indication( int handle, struct sk_buff *skb) {

	struct irlpt_cb *self;

	ASSERT( skb != NULL, return;);

	DEBUG( 3, "irlpt_data_indication: len=%d\n", (int) skb->len);

#if 0
	{
		int i;

		for(i=0;i<skb->len;i++)
			if (skb->data[i] > 31 && skb->data[i] < 128) {
				printk("%c", skb->data[i]);
			} else {
				if (skb->data[i] == 0x0d) {
					printk("\n");
				} else {
					printk(".");
				}
			}

		printk("\n");
	}
#endif
	
	self = find_lmp_handle(handle);
	skb_queue_tail(&self->buffer, skb);
        wake_up_interruptible(&lpt_wait);
}

/*
 * Function irlpt_get_value_confirm (obj_id, type, value_int, value_char, priv)
 *
 *    Fixed to match changes in iriap.h, DB.
 *
 */
void irlpt_get_value_confirm(__u16 obj_id, __u8 type, int value_int, 
			     char *value_char, void *priv)
{
	struct irlpt_frame info;
	struct irlpt_cb *self;

	DEBUG( 3, "irlpt_get_value_confirm()\n");

	self = (struct irlpt_cb *) priv;

	info.dlsap = value_int;
	self->peersap = value_int;

	DEBUG( 3, "irlpt_get_value_confirm: obj_id = %d, value = %d\n", obj_id,
	       value_int);

	ASSERT( type == INTEGER, return;);

	if ( value_int != -1)
		irlpt_do_event( self, IAS_PROVIDER_AVAIL, NULL, &info);
	else
		irlpt_do_event( self, IAS_PROVIDER_NOT_AVAIL, NULL, &info);
}

void irlpt_connect_request( struct irlpt_cb *self) {
	struct notify_cb lpt_notify;

	DEBUG( 3, "irlpt_connect_request()\n");

	self->lsap = NO_HANDLE;

	lpt_notify.connect_confirm = irlpt_connect_confirm;
	lpt_notify.disconnect_indication = irlpt_disconnect_indication;
	lpt_notify.data_indication = irlpt_data_indication;

	self->lsap = irlmp_create_LSAP(LM_ANY, &lpt_notify);
	DEBUG( 3, "irlpt_connect_request: LSAP = %d\n", self->lsap);

	if (self->servicetype == THREE_WIRE_RAW) {
		DEBUG( 3, "irlpt_connect_request: issue THREE_WIRE_RAW connect\n");
		irlmp_connect_request(self->lsap, self->peersap, NULL, NULL);
	}
}

/*
 * Function irlpt_register_server(void)
 *
 *    Register server support so we can accept incomming connections. We
 *    must register both a TSAP for control and data
 * 
 */
void irlpt_register_server(void)
{
	struct notify_cb notify;
	
	DEBUG( 3, "irlpt_register_server()\n");

	/*
	 *  First register control TSAP
	 */

	if ( !irlptd || irlptd->magic != LPT_MAGIC) {
		DEBUG( 0, "irlpt_register_server:, unable to obtain handle!\n");
		return;
	}

	irlptd->lsap = NO_HANDLE;

	notify.connect_confirm = irlpt_connect_confirm;
	notify.connect_indication = irlpt_connect_indication;
	notify.disconnect_indication = irlpt_disconnect_indication;
	notify.data_indication = irlpt_data_indication;

	irlptd->lsap = irlmp_create_LSAP( PRINTER_LSAP, &notify);

	irlptd->connected = WAITING;
	irlptd->service_LSAP = PRINTER_LSAP;

	DEBUG( 3, "irlpt_register_server: LSAP=%d\n", irlptd->lsap);
}
