/*********************************************************************
 *                
 * Filename:      irlpt.h
 * Version:       0.1
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Tue Jun  9 22:33:44 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>
 *     Copyright (c) 1998, Dag Brattli, 
 *     All Rights Reserved
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     I, Thomas Davis, provide no warranty for any of this software. This
 *     material is provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifndef IRCOMM_H
#define IRCOMM_H

#include "qos.h"
#include "irmod.h"

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/miscdevice.h>

/* Command packet types */

#define IRLPT_MAX_PACKET 1024
#define IRLPT_MAX_HEADER LMP_HEADER+LAP_HEADER
#define LPT_MAX_DEVICES 3
#define LPT_MAGIC 0x0755

typedef enum {
	DISCONNECTED,
	WAITING,
	CONNECTED,
	FLUSHED,
} IRLPT_SERVER_STATE;

#define PRINTER_LSAP    0x09

#define PI_SERVICE_TYPE	0x00

#define UNKNOWN         0x00            /* not defined yet. */
#define THREE_WIRE_RAW	0x01		/* bit 0 */
#define THREE_WIRE	0x02		/* bit 1 */
#define NINE_WIRE	0x04		/* bit 2 */
#define CENTRONICS	0x08		/* bit 3 */
#define SERVER_MODE     0xFF            /* our own flag */

#define PI_PORT_TYPE	0x01

#define SERIAL		0x01		/* bit 0 */
#define PARALLEL	0x02		/* bit 1 */

#define PI_PORT_NAME	0x02

#define PI_CRITICAL	0x80


struct irlpt_cb {
        int magic;          /* magic used to detect corruption of the struct */
	__u32 daddr;         /* my local address. */

	struct timer_list retry_timer;
	
	int volatile state; /* Current state of IrCOMM layer */
	int open_retries;
        int in_use;         /* flag to prevent re-use */
        char ifname[9];     /* name of the allocated instance, and registered device. */
	int handle;         /* lmp handle */

	__u8 peersap;       /* remote SAP address */
	__u8 lsap;          /* local SAP address */
	__u8 servicetype;   /* Type of remote service, ie THREE_WIRE_RAW */
	__u8 porttype;      /* type of remote port. */

	struct miscdevice ir_dev; /* used to register the misc device. */

	int count;                /* open count */

	/* this is used by the server side of the system */

        IRLPT_SERVER_STATE connected;
	int eof;
	int service_LSAP;

	struct sk_buff_head buffer; /* buffer queue */

	/*  Used by extract_params, placed here for now to avoid placing
	 *  them on the stack 
	 */
	char name[254];
	char value[1016];
};

/* Debug function */

/* #define CHECK_SKB(skb) check_skb((skb),  __LINE__,__FILE__) */

/* int  irlpt_init( struct device *dev); */
int  irlpt_init(void);
void irlpt_cleanup(void);
struct irlpt_cb *irlpt_open(void);
void irlpt_close(struct irlpt_cb *self);

void irlpt_discovery_indication( __u32 daddr);

void irlpt_data_request( int handle, struct sk_buff *skb);
void irlpt_data_indication( int handle, struct sk_buff *skb);
void irlpt_control_data_indication( int handle, struct sk_buff *skb);

void irlpt_connect_request( struct irlpt_cb *self);
void irlpt_connect(struct irlpt_cb *self, struct sk_buff *skb);
void irlpt_connect_confirm( int handle, struct qos_info *qos, 
			    struct sk_buff *skb);
void irlpt_disconnect_indication( int handle, REASON reason);
void irlpt_flow_indication( int status);

void irlpt_extract_params( struct sk_buff *skb);
void irlpt_get_value_confirm(__u16 obj_id, __u8 type, int value_int, 
			     char *value_char, void *priv);
void irlpt_register_server(void);

extern struct irlpt_cb **irlpt;

#endif
