/*********************************************************************
 *
 * Filename:      irlpt.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 18:54:38 1998
 * Modified at:   Sun Mar  8 23:44:19 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Sources:	  irlan.c
 *
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>,
 *			   Dag Brattli,  <dagb@cs.uit.no>
 *     All Rights Reserved.
 *
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software.
 *     This material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include "ircompat.h"
#include "irlap.h"
#include "irttp.h"
#include "irlmp.h"
#include "iriap.h"
#include "common.h"
#include "server.h"
#include "server_fsm.h"
#include "timer.h"
#include "irda.h"

/* #include "linux/config.h" */
#include "linux/module.h"

/* make depend does not understand LinuxVersionCode(2,1,0) */
#if LINUX_VERSION_CODE > 0x020100 
#include <asm/uaccess.h>
#endif
#include <linux/miscdevice.h>
#include <linux/proc_fs.h>

static struct wait_queue *lpt_wait;
int server_lsap = PRINTER_LSAP;

#if 0
static char *rcsid = "$Id: server.c,v 1.1.1.1 1998/09/29 02:27:14 root Exp $";
#endif
static char *version = "IrLPT server, $Revision: 1.1.1.1 $/$Date: 1998/09/29 02:27:14 $ (Thomas Davis)";

struct file_operations irlpt_fops = {
	seek,		/* seek */
	NULL,
	write,
	NULL,		/* readdir */
	NULL,
	NULL,		/* ioctl */
	NULL,		/* mmap */
	open,
	close,
	NULL,
	NULL,
};

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS

/*
 * Function proc_irlpt_read (buf, start, offset, len, unused)
 *
 *
 *
 */
int server_proc_read(char *buf, char **start, off_t offset, int len, int unused)
{
	int index;

	if (irlptd != NULL) {
	        len = sprintf(buf, "%s\n\n", version);
	        len += sprintf(buf+len, "ifname\tminor\tservicetype\tporttype\tdaddr\tstate\tretries\tpeersap\tcount\tbuffers\n");
	        len += sprintf(buf+len, "----------------------------------------------------------------------------------------------\n");
		len += sprintf(buf+len, "%s\t", irlptd->ifname);
		len += sprintf(buf+len, "%d\t", irlptd->ir_dev.minor);

		switch (irlptd->servicetype) {
		case UNKNOWN:
			index = 0;
			break;
		case THREE_WIRE_RAW:
			index = 1;
			break;
		case THREE_WIRE:
			index = 2;
			break;
		case NINE_WIRE:
			index = 3;
			break;
		case CENTRONICS:
			index = 4;
			break;
		case SERVER_MODE:
			index = 5;
			break;
		default:
			index = 0;
			break;
		}

		len += sprintf(buf+len, "%s\t", service_type[index]);
		len += sprintf(buf+len, "%s\t\t", port_type[irlptd->porttype]);
		len += sprintf(buf+len, "%d\t", irlptd->daddr);
		len += sprintf(buf+len, "%s\t", fsm_state[irlptd->state]);
		len += sprintf(buf+len, "%d\t", irlptd->open_retries);
		len += sprintf(buf+len, "%d\t", irlptd->dlsap_sel);
		len += sprintf(buf+len, "%d\t", irlptd->count);
		len += sprintf(buf+len, "%d", skb_queue_len(&irlptd->buffer));
		len += sprintf(buf+len, "\n");
	}

	return len;
}

extern struct proc_dir_entry proc_irda;

struct proc_dir_entry proc_lpt_server = {
	0, 3, "lpt_server",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&server_proc_read /* get_info */,
};

#endif /* PROC_FS */
#endif /* CONFIG_PROC_FS */

/*
 * Function irlpt_init (dev)
 *
 *   Initializes the irlpt control structure
 *
 */

/*int irlpt_init( struct device *dev) {*/
int server_init(void) {
	DEBUG( 4, "--> server_init\n");

	printk( KERN_INFO "%s\n", version);

	irlptd = (struct irlpt_cb *) kmalloc (sizeof(struct irlpt_cb), GFP_KERNEL);

	if ( irlptd == NULL) {
		printk( KERN_WARNING "IrLPT server: Can't allocate irlptd control block!\n");
		return -ENOMEM;
	}

	memset( irlptd, 0, sizeof(struct irlpt_cb));

	sprintf(irlptd->ifname, "irlptd");
	irlptd->ir_dev.minor = MISC_DYNAMIC_MINOR;
	irlptd->ir_dev.name = irlptd->ifname;
	irlptd->ir_dev.fops = &irlpt_fops;
	misc_register(&irlptd->ir_dev);
	irlptd->magic = LPT_MAGIC;
	irlptd->in_use = TRUE;
	irlptd->servicetype = THREE_WIRE_RAW;
	irlptd->porttype = SERIAL;

	skb_queue_head_init(&irlptd->buffer);

 register_server();

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	proc_register_dynamic( &proc_irda, &proc_lpt_server);
#else
	proc_register( &proc_irda, &proc_lpt_server);
#endif
#endif /* CONFIG_PROC_FS */
#endif /* USE_PROC */

	DEBUG( 4, "server_init -->\n");

	return 0;
}

/*
 * Function irlpt_cleanup (void)
 *
 *
 *
 */
void server_cleanup(void)
{
	struct sk_buff *skb;

	DEBUG( 4, "--> server_cleanup\n");

	while (( skb = skb_dequeue(&irlptd->buffer)) != NULL) {
		DEBUG(3, "server_cleanup: freeing SKB\n");
                IS_SKB( skb, return;);
                FREE_SKB_MAGIC( skb);
                DEV_KFREE_SKB( skb, FREE_READ);
	}

	misc_deregister(&irlptd->ir_dev);

	kfree(irlptd);

#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
	proc_unregister( &proc_irda, proc_lpt_server.low_ino);
#endif
#endif

	DEBUG( 4, "server_cleanup -->\n");
}

/*
 * Function irlpt_close (self)
 *
 *    This function closes and marks the IrLPT instance as not in use.
 */
void server_close( struct irlpt_cb *self)
{
	struct sk_buff *skb;

	DEBUG( 3, "--> irlpt_close()\n");

	ASSERT( self != NULL, return;);

	/* First make sure we're connected. */
	if ( !self || self->magic != LPT_MAGIC) {
		DEBUG( 0, "irlpt_close: bad pointer *self or bad magic!\n");
		return;
	}

	while (( skb = skb_dequeue(&self->buffer)) != NULL) {
		DEBUG(3, "irlpt_close: freeing SKB\n");
                IS_SKB( skb, return;);
                FREE_SKB_MAGIC( skb);
                DEV_KFREE_SKB( skb, FREE_READ);
	}

	misc_deregister(&self->ir_dev);

	self->in_use = FALSE;

	/* Clear structure */
	memset( self, 0, sizeof(struct irlpt_cb));

	/* Set magic since the structure exists (is allocated) */
	self->magic = LPT_MAGIC;

	/* MOD_DEC_USE_COUNT; */

	DEBUG( 3, "irlpt_close() -->\n");
}

/*
 * Function irlpt_disconnect_indication (handle)
 *
 */
void server_disconnect_indication( void *instance, void *sap, LM_REASON reason,
				   struct sk_buff *userdata)
{
	struct irlpt_info info;
	struct irlpt_cb *self;

	self = ( struct irlpt_cb *) instance;

	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	info.daddr = self->daddr;

	DEBUG( 3, "server_disconnect_indication, reason=%d (%s), dlsap_sel=%d\n",
	       reason, reasons[reason], self->dlsap_sel);

	self->connected = DISCONNECTED;
	self->eof = reason;

        wake_up_interruptible(&lpt_wait);

	DEBUG( 3, "irlpt_disconnect_indication: skb_queue_len=%d\n",
	       skb_queue_len(&irlptd->buffer));

	server_do_event( self, LMP_DISCONNECT, NULL, NULL);

	register_server();
}

/*
 * Function irlpt_connect_confirm (handle, qos, skb)
 *
 *    LSAP connection confirmed!
 */
void server_connect_confirm( void *instance, void *sap, struct qos_info *qos,
			     struct sk_buff *skb)
{
	struct irlpt_info info;
	struct irlpt_cb *self;

	self = ( struct irlpt_cb *) instance;

	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	info.daddr = self->daddr;

	DEBUG( 3, "server_connect_confirmation\n");

	/*
	 *  Check if we have got some QoS parameters back! This should be the
	 *  negotiated QoS for the link.
	 */
	if ( qos) {
		DEBUG( 0, "IrLPT, Negotiated BAUD_RATE: %02x\n", 
		       qos->baud_rate.bits);			
		DEBUG( 0, "IrLPT, Negotiated BAUD_RATE: %d bps.\n", 
		       qos->baud_rate.value);
	}

	self->connected = TRUE;

	server_do_event( self, LMP_CONNECT, NULL, NULL);

}

/*
 * Function irlpt_connect_indication (handle)
 *
 */
void server_connect_indication( void *instance, void *sap, 
				struct qos_info *qos, struct sk_buff *skb)
{
	struct irlpt_cb *self;
	struct irlpt_info info;
	struct lsap_cb *lsap;

	DEBUG( 3, "server_connect_indication()\n");

	self = ( struct irlpt_cb *) instance;
	lsap = (struct lsap_cb *) sap;
	
	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	self->connected = CONNECTED;
	self->eof = FALSE;

	info.lsap = lsap;

	server_do_event( self, LMP_CONNECT, NULL, &info);
}

/*
 * Function irlpt_data_indication (handle, skb)
 *
 *    This function gets the data that is received on the data channel
 *
 */
void server_data_indication( void *instance, void *sap, struct sk_buff *skb) {

	struct irlpt_cb *self;

	self = ( struct irlpt_cb *) instance;
	     
	ASSERT( self != NULL, return;);
	ASSERT( self->magic == LPT_MAGIC, return;);

	ASSERT( skb != NULL, return;);

	DEBUG( 3, "server_data_indication: len=%d\n", (int) skb->len);

#if 0
	dump_buffer(skb);
#endif
	
	skb_queue_tail(&self->buffer, skb);
        wake_up_interruptible(&lpt_wait);
}

/*
 * Function register_server(void)
 *
 *    Register server support so we can accept incomming connections. We
 *    must register both a TSAP for control and data
 * 
 */
void register_server(void)
{
	struct notify_t notify;
	
	DEBUG( 3, "register_server()\n");

	/*
	 *  First register control TSAP
	 */

	if ( !irlptd || irlptd->magic != LPT_MAGIC) {
		DEBUG( 0, "register_server:, unable to obtain handle!\n");
		return;
	}

	notify.connect_confirm = server_connect_confirm;
	notify.connect_indication = server_connect_indication;
	notify.disconnect_indication = server_disconnect_indication;
	notify.data_indication = server_data_indication;

	irlptd->lsap = irlmp_open_lsap( server_lsap, &notify);

	irlptd->connected = WAITING;
	irlptd->service_LSAP = server_lsap;

	DEBUG( 3, "register_server: Source LSAP sel=%d\n", irlptd->slsap_sel);
}

#ifdef MODULE

#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
MODULE_AUTHOR("Thomas Davis <ratbert@radiks.net>");
MODULE_DESCRIPTION("The Linux IrDA/IrLPT protocol");
#endif

/*
 * Function init_module (void)
 *
 *    Initialize the IrLPT server module, this function is called by the
 *    modprobe(1) program.
 */
int init_module(void)
{

        DEBUG( 4, "--> irlpt server: init_module\n");

        server_init();

        DEBUG( 4, "irlpt server: init_module -->\n");

        return 0;
}

/*
 * Function cleanup_module (void)
 *
 *    Remove the IrLPT server module, this function is called by the rmmod(1)
 *    program
 */
void cleanup_module(void)
{
        DEBUG( 4, "--> irlpt server: cleanup_module\n");
        /* No need to check MOD_IN_USE, as sys_delete_module() checks. */

        /* Free some memory */
        server_cleanup();

        DEBUG( 4, "irlpt server: cleanup_module -->\n");
}

#endif /* MODULE */
