/*********************************************************************
 *                
 * Filename:      server_fsm.c
 * Version:       0.1
 * Sources:       irlan_event.c
 * 
 *     Copyright (c) 1997, Dag Brattli <dagb@cs.uit.no>, All Rights Reserved.
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>, All Rights Reserved.
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *
 *     I, Thomas Davis, provide no warranty for any of this software. This 
 *     material is provided "AS-IS" and at no charge.
 *
 ********************************************************************/

#include "ircompat.h"
#include "iriap.h"
#include "irlmp.h"
#include "irttp.h"
#include "common.h" 
#include "server.h"
#include "server_fsm.h"
#include "irda.h"

static int server_state_idle  ( struct irlpt_cb *self, IRLPT_EVENT event,
				struct sk_buff *skb, 
				struct irlpt_info *info);
static int server_state_conn  ( struct irlpt_cb *self, IRLPT_EVENT event, 
				struct sk_buff *skb, 
				struct irlpt_info *info);

#if 0
static char *rcsid = "$Id: server_fsm.c,v 1.1.1.1 1998/09/29 02:27:14 root Exp $";
#endif

static int (*state[])( struct irlpt_cb *self, IRLPT_EVENT event, 
		       struct sk_buff *skb,
		       struct irlpt_info *info) = 
{ 
	server_state_idle,
	server_state_conn,
};

void server_do_event( struct irlpt_cb *self, IRLPT_EVENT event, 
		      struct sk_buff *skb, struct irlpt_info *info) {
	DEBUG( 4, "server_do_event: STATE = %s, EVENT = %s\n", 
	       fsm_state[self->state], fsm_event[event]);
	(*state[ self->state]) ( self, event, skb, info);
}

void server_next_state( struct irlpt_cb *self, IRLPT_STATE state) {
	DEBUG( 4, "server_next_state: NEXT STATE = %s\n", fsm_state[state]);
	self->state = state;
}

/*
 * Function server_state_idle (event, skb, info)
 *
 *    IDLE, We are waiting for an indication that there is a provider
 *    available.
 */
static int server_state_idle( struct irlpt_cb *self, IRLPT_EVENT event, 
			     struct sk_buff *skb, 
			     struct irlpt_info *info) {

	struct sk_buff *r_skb;

	DEBUG( 4, "server_state_idle()\n");

	switch( event) {
	case LMP_CONNECT:
		DEBUG( 3, "server_state_idle: LM_CONNECT, remote lsap=%d\n", 
		       info->dlsap_sel);

		self->dlsap_sel = info->dlsap_sel;

		r_skb = dev_alloc_skb(64);
		if (r_skb == NULL) { 
			DEBUG( 0, "couldn't allocate an sk_buff of length 64\n");
			return 0;
		}
		ALLOC_SKB_MAGIC(r_skb);
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
		r_skb->free=1;
#endif
		skb_reserve( r_skb, LMP_MAX_HEADER);
		skb->len = 0;
		irlmp_connect_response( self->lsap, r_skb);
		server_next_state( self, LPT_CONN);

		break;

	default:
		DEBUG( 0, "server_state_idle(), Unknown event %d\n", event);
		break;
	}

	if (skb) {
		DEV_KFREE_SKB( skb, FREE_READ);
	}
	return 0;
}

/*
 * Function server_state_conn (event, skb, info)
 *
 *    CONN, We have connected to a provider but has not issued any
 *    commands yet.
 *
 */
static int server_state_conn( struct irlpt_cb *self, IRLPT_EVENT event, 
			     struct sk_buff *skb, struct irlpt_info *info) {
	DEBUG( 4, "server_state_conn()\n");

	switch( event) {
	case LMP_DISCONNECT:
	case LAP_DISCONNECT:
		DEBUG( 4, "server_state_conn: LMP_DISCONNECT/LAP_DISCONNECT\n");
		server_next_state( self, LPT_IDLE);
		break;

	default:
		DEBUG( 4, "server_state_conn: Unknown event %d\n", event);
		break;
	}
	if ( skb) {
		DEV_KFREE_SKB( skb, FREE_READ);
	}
	return 0;
}

void server_print_event( IRLPT_EVENT event) {
	DEBUG( 0, "IRLPT_EVENT = %s\n", fsm_event[event]);
}


