/*======================================================================

    The contents of this file are subject to the Mozilla Public
    License Version 1.0 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.

    The initial developer of the original code is David A. Hinds
    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
    
======================================================================*/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <syslog.h>
#include <getopt.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <sys/file.h>
#include <sys/kd.h>



int be_quiet = 0;

/*
 * Function beep (ms, freq)
 *
 *    
 *
 */
void beep(unsigned int ms, unsigned int freq)
{
	int fd, arg;
	
	if (be_quiet)
		return;
	fd = open("/dev/console", O_RDWR);
	if (fd < 0)
		return;
	arg = (ms << 16) | freq;
	ioctl(fd, KDMKTONE, arg);
	close(fd);
	usleep(ms*1000);
}

/*
 * Function open_dev (dev, mode)
 *
 *    Open a device
 *
 */
int open_dev(dev_t dev, int mode)
{
	char *fn;
	int fd;
	if ((fn = tmpnam(NULL)) == NULL)
		return -1;
	if (mknod(fn, mode, dev) != 0)
		return -1;
	fd = open(fn, (mode & S_IWRITE) ? O_RDWR: O_RDONLY);
	unlink(fn);
	return fd;
}

/*
 * Function lookup_dev (name)
 *
 *    Look up the minor number of the IrDA device
 *
 */
int lookup_dev(char *name)
{
	FILE *f;
	int n;
	char s[32], t[32];
	
	f = fopen("/proc/misc", "r");
	if (f == NULL)
		return -errno;
	while (fgets(s, 32, f) != NULL) {
		if (sscanf(s, "%d %s", &n, t) == 2)
			if (strcmp(name, t) == 0)
				break;
	}
	fclose(f);
	if (strcmp(name, t) == 0)
		return n;
	else
		return -ENODEV;
}

int execute(char *msg, char *cmd)
{
	int ret;
	FILE *f;
	char line[256];

	syslog(LOG_INFO, "executing: '%s'", cmd);
	strcat(cmd, " 2>&1");
	f = popen(cmd, "r");
	while (fgets(line, 255, f)) {
		line[strlen(line)-1] = '\0';
		syslog(LOG_INFO, "+ %s", line);
	}
	ret = pclose(f);
	if (WIFEXITED(ret)) {
		if (WEXITSTATUS(ret))
			syslog(LOG_INFO, "%s exited with status %d",
			       msg, WEXITSTATUS(ret));
		return WEXITSTATUS(ret);
	} else
		syslog(LOG_INFO, "%s exited on signal %d",
		       msg, WTERMSIG(ret));
	return -1;
}

int execute_on_dev(char *action, char *class, char *dev)
{
	char msg[128], cmd[512];

	sprintf(msg, "%s cmd", action);
	sprintf(cmd, "./%s %s %s", class, action, dev);
	return execute(msg, cmd);
}





