/*********************************************************************
 *                
 * Filename:      irobex_lib.h
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Mon Jul 20 22:28:23 1998
 * Modified at:   Tue Sep 22 22:28:43 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998 Dag Brattli, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifndef IROBEX_APP_H
#define IROBEX_APP_H

/* use 0 for production, 1 for verification, >2 for debug */
#ifndef NET_DEBUG
#define NET_DEBUG 3
static unsigned int net_debug = NET_DEBUG;
#define DEBUG(n, args...) if (net_debug >= (n)) printf(args)
#endif /* NET_DEBUG */

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE 
#define FALSE 0
#endif

#define PALM_III

#define OBEX_VERSION 0x10 /* Version 1.0 */

/*
 *  I'm just to used with these ones :-)
 */
#define __u8  unsigned char
#define __u16 unsigned short int
#define __u32 unsigned int

/* Standard headers */
#define HEADER_COUNT       0xc0
#define HEADER_NAME        0x01
#define HEADER_TYPE        0x42
#define HEADER_TIME        0x44
#define HEADER_LENGTH      0xc3
#define HEADER_DESCRIPTION 0x05
#define HEADER_BODY        0x48
#define HEADER_BODY_END    0x49

/* Application defined headers */
#define HEADER_CREATOR_ID  0xcf

#define FINAL          0x80

/* Commands */
#define CMD_CONNECT    0x00
#define CMD_DISCONNECT 0x01
#define CMD_PUT        0x02
#define CMD_GET        0x03
#define CMD_COMMAND    0x04
#define CMD_SETPATH    0x05
#define CMD_ABORT      0x7f

#define	RSP_CONTINUE   0x10
#define RSP_SWITCH_PRO 0x11
#define RSP_SUCCESS    0x20
#define RSP_CREATED    0x21
#define RSP_ACCEPTED   0x22
#define RSP_CONFLICT   0x49

#define ADDRESS_BOOK 0x72646461 /* "rdda" */
#define TO_DO_LIST   0x6f646f74 /* "odot" */
#define MEMO_PAD     0x6f6d656d /* "omem" */
#define DATE_BOOK    0x65746164

struct irobex_ioc_t {
     __u32 daddr;
};

#define IROBEX_IOC_MAGIC 'k'

#define IROBEX_IOCSCONNECT    _IOW(IROBEX_IOC_MAGIC, 1, 4)
#define IROBEX_IOCSDISCONNECT _IOW(IROBEX_IOC_MAGIC, 2, 4)
#define IROBEX_IOC_MAXNR 2

/* IrOBEX states */
typedef enum {
	STATE_DISCONNECTED,
	STATE_CONNECTING,
	STATE_READY,
	STATE_PUT,
	STATE_GET,
} IROBEX_STATE;

/* State of the PUT command */
typedef enum {
	STATE_PUT_IDLE,
	STATE_PUT_HEADERS,
	STATE_PUT_BODY,
	STATE_PUT_BODY_END,
} PUT_STATE;

/* IrOBEX events */
typedef enum {
	CONNECT_INDICATION,
	CONNECT_REQUEST,
	DISCONNECT_INDICATION,
	DISCONNECT_REQUEST,
	GET_REQUEST,
	GET_INDICATION,
	PUT_REQUEST,
	PUT_INDICATION,
	RESPONSE_INDICATION,
} IROBEX_EVENT;

typedef struct {
	char name[64];
	char type[64];
	char description[64];
	int length;
	__u32 creator_id;

	__u8 *body;
	int index;
} IROBEX_OBJECT;

typedef void (*OBJECT_INDICATION_CALLBACK)( IROBEX_OBJECT *object);
typedef void (*PROGRESS_CALLBACK)(int ratio);

typedef struct {
	volatile int state;
	volatile int put_state; /* Current state during a put command */

	int mtu; /* Maximum IrOBEX packet size */

	int fd_dev;
 	int fd_file; 
	
	int daddr; /* Destination device address (Not actually used yet!) */

	IROBEX_OBJECT *object;

	OBJECT_INDICATION_CALLBACK callback;
	PROGRESS_CALLBACK progress_callback;
} IROBEX_HANDLE;

IROBEX_HANDLE *irobex_init( OBJECT_INDICATION_CALLBACK callback);
void irobex_put_file( IROBEX_HANDLE *self, char *filename);
void irobex_get_file( IROBEX_HANDLE *self);

#endif

