/*********************************************************************
 *                
 * Filename:      irproc.c
 * Version:       
 * Description:   Various entries in the /proc file system
 * Status:        Experimental.
 * Author:        Thomas Davis, <ratbert@radiks.net>
 * Created at:    Sat Feb 21 21:33:24 1998
 * Modified at:   Mon Oct  5 22:34:34 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998, Thomas Davis, <ratbert@radiks.net>, 
 *     All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     I, Thomas Davis, provide no warranty for any of this software. 
 *     This material is provided "AS-IS" and at no charge. 
 *     
 *     Portions lifted from the linux/fs/procfs/ files.
 *
 ********************************************************************/

#ifdef USE_PROC

#include <linux/miscdevice.h>
#include <linux/proc_fs.h>

#include "irmod.h"
#include "ircompat.h"
#include "irlap.h"
#include "irlmp.h"

#if 0
static char *rcsid = "$Id: irproc.c,v 1.3 1998/10/24 10:04:09 ryan Exp $";

static char *irproc_version = "irproc, $Revision: 1.3 $/$Date: 1998/10/24 10:04:09 $";
#endif

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static int proc_irda_20x_lookup(struct inode *dir,const char *name, int len,
				struct inode ** result);
#else
static int proc_irda_21x_lookup(struct inode * dir, struct dentry *dentry);
#endif

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static int proc_irda_readdir(struct inode * inode, struct file *filp,
			     void * dirent, filldir_t filldir);
#else
static int proc_irda_readdir(struct file *filp, void *dirent, 
 			     filldir_t filldir);
#endif 

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static struct inode *proc_get_inode( struct super_block *s, int ino, 
 				     struct proc_dir_entry *de);
static int proc_irda_match( int len, const char *name, 
			    struct proc_dir_entry *de);
#endif

#if 0
extern int irlpt_proc_read( char *buf, char **start, off_t offset, int len, 
			    int unused);
#endif

extern int driver_proc_read( char *buf, char **start, off_t offset, int len, 
			     int unused);
extern int irlap_proc_read( char *buf, char **start, off_t offset, int len, 
			    int unused);
extern int irlmp_proc_read( char *buf, char **start, off_t offset, int len, 
			    int unused);
extern int irttp_proc_read( char *buf, char **start, off_t offset, int len, 
			    int unused);
extern int irias_proc_read( char *buf, char **start, off_t offset, int len,
 			    int unused);

static int proc_discovery_read( char *buf, char **start, off_t offset, int len,
				int unused);

/* int proc_irda_readdir(struct inode *inode, struct file *filp, void *dirent,  */
/* 		      filldir_t filldir); */

enum irda_directory_inos {
	PROC_IRDA_LAP = 1,
	PROC_IRDA_LMP,
	PROC_IRDA_TTP,
	PROC_IRDA_LPT,
	PROC_IRDA_COMM,
 	PROC_IRDA_DRIVER,
	PROC_IRDA_IRIAS
};

static struct file_operations proc_irda_dir_operations = {
        NULL,                   /* lseek - default */
        NULL,                   /* read - bad */
        NULL,                   /* write - bad */
        proc_irda_readdir,      /* readdir */
        NULL,                   /* select - default */
        NULL,                   /* ioctl - default */
        NULL,                   /* mmap */
        NULL,                   /* no special open code */
        NULL,                   /* no special release code */
        NULL                    /* can't fsync */
};

/*
 * proc directories can do almost nothing..
 */
struct inode_operations proc_irda_dir_inode_operations = {
        &proc_irda_dir_operations,   /* default net directory file-ops */
        NULL,                   /* create */
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
        proc_irda_20x_lookup,            /* lookup */
#else
	proc_irda_21x_lookup, 
#endif
        NULL,                   /* link */
        NULL,                   /* unlink */
        NULL,                   /* symlink */
        NULL,                   /* mkdir */
        NULL,                   /* rmdir */
        NULL,                   /* mknod */
        NULL,                   /* rename */ 
        NULL,                   /* readlink */
        NULL,                   /* follow_link */
        NULL,                   /* readpage */
        NULL,                   /* writepage */
        NULL,                   /* bmap */
        NULL,                   /* truncate */   
        NULL                    /* permission */
};

struct proc_dir_entry proc_irda = {
	0, 4, "irda",
        S_IFDIR | S_IRUGO | S_IXUGO, 2, 0, 0,
        0, &proc_irda_dir_inode_operations,
        NULL, NULL,
        NULL,
        NULL, NULL
};

#if 0 
struct proc_dir_entry proc_lpt = {
	0, 3, "lpt",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&irlpt_proc_read /* get_info */,
};
#endif

struct proc_dir_entry proc_discovery = {
	0, 9, "discovery",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&proc_discovery_read /* get_info */,
};

struct proc_dir_entry proc_driver = {
	0, 6, "driver",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL,
	&driver_proc_read,
};

struct proc_dir_entry proc_ttp = {
	0, 5, "irttp",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&irttp_proc_read /* get_info */,
};

struct proc_dir_entry proc_lmp = {
	0, 5, "irlmp",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&irlmp_proc_read /* get_info */,
};

struct proc_dir_entry proc_lap = {
	0, 5, "irlap",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&irlap_proc_read /* get_info */,
};

struct proc_dir_entry proc_ias = {
	0, 5, "irias",
	S_IFREG | S_IRUGO, 1, 0, 0,
	0, NULL /* ops -- default to array */,
	&irias_proc_read /* get_info */,
};

#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
/*
 * Function proc_delete_dentry (dentry)
 *
 *    Copy of proc/root.c because this function is invisible to the irda
 *    module
 * 
 */

static void proc_delete_dentry(struct dentry * dentry)
{
	d_drop(dentry);
}

static struct dentry_operations proc_dentry_operations =
{
	NULL,			/* revalidate */
	NULL,			/* d_hash */
	NULL,			/* d_compare */
	proc_delete_dentry	/* d_delete(struct dentry *) */
};
#endif

/*
 * Function irda_proc_register (void)
 *
 *    Register irda entry in /proc file system
 *
 */
void irda_proc_register(void) {
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
 	proc_register_dynamic( &proc_net,  &proc_irda);
	proc_register_dynamic( &proc_irda, &proc_lap);
	proc_register_dynamic( &proc_irda, &proc_lmp);
	proc_register_dynamic( &proc_irda, &proc_ttp);
	proc_register_dynamic( &proc_irda, &proc_ias);
	proc_register_dynamic( &proc_irda, &proc_driver);
	proc_register_dynamic( &proc_irda, &proc_discovery);
#else
	proc_net_register( &proc_irda);
	proc_register( &proc_irda, &proc_lap);
	proc_register( &proc_irda, &proc_lmp);
	proc_register( &proc_irda, &proc_ttp);
	proc_register( &proc_irda, &proc_ias);
 	proc_register( &proc_irda, &proc_driver); 
	proc_register( &proc_irda, &proc_discovery);
#endif
}

/*
 * Function irda_proc_unregister (void)
 *
 *    Unregister irda entry in /proc file system
 *
 */
void irda_proc_unregister(void) {
	proc_unregister( &proc_irda, proc_discovery.low_ino);
 	proc_unregister(&proc_irda, proc_driver.low_ino);
	proc_unregister( &proc_irda, proc_ias.low_ino);
	proc_unregister( &proc_irda, proc_ttp.low_ino);
	proc_unregister( &proc_irda, proc_lmp.low_ino);
	proc_unregister( &proc_irda, proc_lap.low_ino);
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	proc_unregister( &proc_net, proc_irda.low_ino);
#else
	proc_unregister( proc_net, proc_irda.low_ino);
#endif
}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static int proc_irda_match( int len, const char * name, 
			    struct proc_dir_entry * de)
{
	if (!de || !de->low_ino)
		return 0;
 	/* "" means "." ---> so paths like "/usr/lib//libc.a" work */
	if (!len && (de->name[0]=='.') && (de->name[1]=='\0'))
		return 1;
	if (de->namelen != len)
		return 0;
	return !memcmp(name, de->name, len);
}
#endif

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static struct inode *proc_get_inode(struct super_block * s, int ino,
 				     struct proc_dir_entry * de)
{
	struct inode * inode = iget(s, ino);
	if (inode && inode->i_sb == s) {
		inode->u.generic_ip = (void *) de;
		if (de) { 
			if (de->mode) {
				inode->i_mode = de->mode;
				inode->i_uid = de->uid;
				inode->i_gid = de->gid; 
			} 
			if (de->size) 
				inode->i_size = de->size; 
			if (de->ops) 
				inode->i_op = de->ops; 
			if (de->nlink) 
				inode->i_nlink = de->nlink; 
			if (de->fill_inode) 
				de->fill_inode(inode); 
		}
	}
	return inode;
}

static int proc_irda_20x_lookup(struct inode * dir,const char * name, int len,
				struct inode ** result)
{
	struct proc_dir_entry * de;
	int ino;

	*result = NULL;
	if (!dir || !S_ISDIR(dir->i_mode)) {
		iput(dir);
		return -ENOTDIR;
	}

	de = (struct proc_dir_entry *) dir->u.generic_ip;
	if (!de) {
		iput(dir);
		return -EINVAL;
	}

	/* Special case "." and "..": they aren't on the directory list */
	*result = dir;
	if (!len)
		return 0;
	if (name[0] == '.') {
		if (len == 1)
			return 0;
		if (name[1] == '.' && len == 2) {
			struct inode * inode;
			inode = proc_get_inode(dir->i_sb, de->parent->low_ino,
					       de->parent);
			iput(dir);
			if (!inode)
				return -EINVAL;
			*result = inode;
			return 0;
		}
	}

	*result = NULL;
	for (de = de->subdir; de ; de = de->next) {
		if (proc_irda_match(len, name, de))
			break;
	}
	if (!de) {
		iput(dir);
		return -ENOENT;
	}
	
	ino = de->low_ino | (dir->i_ino & ~(0xffff));

	if (!(*result = proc_get_inode(dir->i_sb, ino, de))) {
		iput(dir);
		return -EINVAL;
	}
	iput(dir);
	return 0;
}
#else
/*
 * Function proc_irda_21x_lookup (dir, dentry)
 *
 *    This is a copy of proc_lookup from the linux-2.1.x 
 *
 */
int proc_irda_21x_lookup(struct inode * dir, struct dentry *dentry)
{
	struct inode *inode;
	struct proc_dir_entry * de;
	int error;

	error = -ENOTDIR;
	if (!dir || !S_ISDIR(dir->i_mode))
		goto out;

	error = -ENOENT;
	inode = NULL;
	de = (struct proc_dir_entry *) dir->u.generic_ip;
	if (de) {
		for (de = de->subdir; de ; de = de->next) {
			if (!de || !de->low_ino)
				continue;
			if (de->namelen != dentry->d_name.len)
				continue;
			if (!memcmp(dentry->d_name.name, de->name, de->namelen)) {
				int ino = de->low_ino | (dir->i_ino & ~(0xffff));
				error = -EINVAL;
				inode = proc_get_inode(dir->i_sb, ino, de);
				break;
			}
		}
	}

	if (inode) {
		dentry->d_op = &proc_dentry_operations;
		d_add(dentry, inode);
		error = 0;
	}
out:
	return error;
}
#endif

/*
 * Function proc_irda_readdir (filp, dirent, filldir)
 *
 *    This is a copy from linux/fs/proc because the function is invisible
 *    to the irda module
 * 
 */
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
static int proc_irda_readdir( struct inode * inode, struct file * filp,
			      void * dirent, filldir_t filldir)
#else
static int proc_irda_readdir( struct file *filp, void *dirent, 
			      filldir_t filldir)
#endif 

{
	struct proc_dir_entry * de;
	unsigned int ino;
	int i;
	
#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
	struct inode *inode = filp->f_dentry->d_inode;
#endif
	if (!inode || !S_ISDIR(inode->i_mode))
		return -ENOTDIR;
	ino = inode->i_ino;
	de = (struct proc_dir_entry *) inode->u.generic_ip;
	if (!de)
		return -EINVAL;
	i = filp->f_pos;
	switch (i) {
	case 0:
		if (filldir(dirent, ".", 1, i, ino) < 0)
			return 0;
		i++;
		filp->f_pos++;
		/* fall through */
	case 1:
		if (filldir(dirent, "..", 2, i, de->parent->low_ino) < 0)
			return 0;
		i++;
		filp->f_pos++;
		/* fall through */
	default:
		ino &= ~0xffff;
		de = de->subdir;
		i -= 2;
		for (;;) {
			if (!de)
				return 1;
			if (!i)
				break;
			de = de->next;
			i--;
		}
		
		do {
			if (filldir(dirent, de->name, de->namelen, filp->f_pos,
				    ino | de->low_ino) < 0)
				return 0;
			filp->f_pos++;
			de = de->next;
		} while (de);
	}
	return 1;
}

#if 0
static char *link_status[] = {
	"OK",
	"MEDIA_BUSY",
	"ABORTED",
	"NO_ACTIVITY",
	"NOISY",
	"REMOTE",
};
#endif

/*
 * Function proc_discovery_read (buf, start, offset, len, unused)
 *
 *    Print discovery information in /proc file system
 *
 */
int proc_discovery_read( char *buf, char **start, off_t offset, int len, 
			 int unused)
{
	DISCOVERY *discovery;

	if ( !irlmp)
		return len;

	len = sprintf(buf, "IrLMP: Discovery log:\n\n");	

/* 	discovery = ( DISCOVERY *) hashbin_get_first( irlmp->discovery_log); */
/* 	while ( discovery != NULL) { */
/* 		len += sprintf( buf+len, "Name: %s\n", discovery->info); */

/* 		len += sprintf( buf+len, "Hint: "); */
/* 		if ( discovery->hint[0] & PNP) */
/* 			len += sprintf( buf+len, "PnP Compatible,"); */
/* 		if ( discovery->hint[0] & PDA) */
/* 			len += sprintf( buf+len, "PDA/Palmtop\n"); */
/* 		if ( discovery->hint[0] & COMPUTER) */
/* 			len += sprintf( buf+len, "Computer\n"); */
/* 		if ( discovery->hint[0] & PRINTER) */
/* 			len += sprintf( buf+len, "Printer\n"); */
/* 		if ( discovery->hint[0] & MODEM) */
/* 			len += sprintf( buf+len, "Modem "); */
/* 		if ( discovery->hint[0] & FAX) */
/* 			len += sprintf( buf+len, "Fax "); */
/* 		if ( discovery->hint[0] & LAN) */
/* 			len += sprintf( buf+len, "LAN Access\n"); */

/* 		if ( discovery->hint[1] & TELEPHONY) */
/* 			len += sprintf( buf+len, "Telephony "); */
/* 		if ( discovery->hint[1] & FILE_SERVER) */
/* 			len += sprintf( buf+len, "File Server ");        */
/* 		if ( discovery->hint[1] & IRCOMM) */
/* 			len += sprintf( buf+len, "IrCOMM "); */

/* 		len += sprintf( buf+len, "Dest addr: 0x%08x\n",  */
/* 				discovery->daddr); */
		
/* 		len += sprintf( buf+len, "\n\n"); */

/* 		discovery = ( DISCOVERY *) hashbin_get_next( irlmp->discovery_log); */
/* 	} */
	
	return len;
}

#endif /* USE_PROC */
