/*********************************************************************
 *                
 * Filename:      irsysctl.c
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sun May 24 22:12:06 1998
 * Modified at:   Sun Oct 18 08:28:17 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1997 Dag Brattli, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifdef USE_SYSCTL

#include <linux/mm.h>
#include <linux/ctype.h>
#include <linux/sysctl.h>
#include <asm/segment.h>

#include "ircompat.h"

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,8)
int proc_dointvec(ctl_table *table, int write, struct file *filp,
                  void *buffer, size_t *lenp);
int proc_dostring(ctl_table *table, int write, struct file *filp,
                  void *buffer, size_t *lenp);
int sysctl_string(ctl_table *table, int *name, int nlen,
                  void *oldval, size_t *oldlenp,
                  void *newval, size_t newlen, void **context);
#endif

#define NET_IRDA 412 /* Random number */
enum { DISCOVERY=1, DEVNAME, COMPRESSION };

int sysctl_discovery = 0;
int sysctl_compression = 0;
char sysctl_devname[65];

/* One file */
static ctl_table irda_table[] = {
	{ DISCOVERY, "discovery", &sysctl_discovery,
	  sizeof(int), 0644, NULL, &proc_dointvec },
	{ DEVNAME, "devname", sysctl_devname,
	  65, 0644, NULL, &proc_dostring, &sysctl_string},
	{ COMPRESSION, "compression", &sysctl_compression,
	  sizeof(int), 0644, NULL, &proc_dointvec },
	{ 0 }
};

/* One directory */
static ctl_table irda_net_table[] = {
	{ NET_IRDA, "irda", NULL, 0, 0555, irda_table },
	{ 0 }
};

/* The parent directory */
static ctl_table irda_root_table[] = {
	{ CTL_NET, "net", NULL, 0, 0555, irda_net_table },
	{ 0 }
};

static struct ctl_table_header *irda_table_header;

/*
 * Function irda_sysctl_register (void)
 *
 *    Register our sysctl interface
 *
 */
int irda_sysctl_register(void)
{
	irda_table_header = register_sysctl_table( irda_root_table, 0);
	if ( !irda_table_header)
		return -ENOMEM;
	return 0;
}

/*
 * Function irda_sysctl_unregister (void)
 *
 *    Unregister our sysctl interface
 *
 */
void irda_sysctl_unregister(void) 
{
	unregister_sysctl_table( irda_table_header);
}

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,8)
int proc_dointvec(ctl_table *table, int write, struct file *filp,
                  void *buffer, size_t *lenp)
{
        int *i, vleft, first=1, len, left, neg, val;
        #define TMPBUFLEN 20
        char buf[TMPBUFLEN], *p;
        
        if (!table->data || !table->maxlen || !*lenp ||
            (filp->f_pos && !write)) {
                *lenp = 0;
                return 0;
        }
        
        i = (int *) table->data;
        vleft = table->maxlen / sizeof(int);
        left = *lenp;
        
        for (; left && vleft--; i++, first=0) {
                if (write) {
                        while (left && isspace(get_user((char *) buffer)))
                                left--, ((char *) buffer)++;
                        if (!left)
                                break;
                        neg = 0;
                        len = left;
                        if (len > TMPBUFLEN-1)
                                len = TMPBUFLEN-1;
                        memcpy_fromfs(buf, buffer, len);
                        buf[len] = 0;
                        p = buf;
                        if (*p == '-' && left > 1) {
                                neg = 1;
                                left--, p++;
                        }
                        if (*p < '0' || *p > '9')
                                break;
                        val = simple_strtoul(p, &p, 0);
                        len = p-buf;
                        if ((len < left) && *p && !isspace(*p))
                                break;
                        if (neg)
                                val = -val;
                        buffer += len;
                        left -= len;
                        *i = val;
                } else {
                        p = buf;
                        if (!first)
                                *p++ = '\t';
                        sprintf(p, "%d", *i);
                        len = strlen(buf);
                        if (len > left)
                                len = left;
                        memcpy_tofs(buffer, buf, len);
                        left -= len;
                        buffer += len;
                }
        }

        if (!write && !first && left) {
                put_user('\n', (char *) buffer);
                left--, buffer++;
        }
        if (write) {
                p = (char *) buffer;
                while (left && isspace(get_user(p++)))
                        left--;
        }
        if (write && first)
                return -EINVAL;
        *lenp -= left;
        filp->f_pos += *lenp;
        return 0;
}

int proc_dostring(ctl_table *table, int write, struct file *filp,
                  void *buffer, size_t *lenp)
{
        int len;
        char *p, c;
        
        if (!table->data || !table->maxlen || !*lenp ||
            (filp->f_pos && !write)) {
                *lenp = 0;
                return 0;
        }
        
        if (write) {
                len = 0;
                p = buffer;
                while (len < *lenp && 
                       (c = get_user(p++)) != 0 && c != '\n')
                        len++;
                if (len >= table->maxlen)
                        len = table->maxlen-1;
                memcpy_fromfs(table->data, buffer, len);
                ((char *) table->data)[len] = 0;
                filp->f_pos += *lenp;
        } else {
                len = strlen(table->data);
                if (len > table->maxlen)
                        len = table->maxlen;
                if (len > *lenp)
                        len = *lenp;
                if (len)
                        memcpy_tofs(buffer, table->data, len);
                if (len < *lenp) {
                        put_user('\n', ((char *) buffer) + len);
                        len++;
                }
                *lenp = len;
                filp->f_pos += len;
        }
        return 0;
}

int sysctl_string(ctl_table *table, int *name, int nlen,
                  void *oldval, size_t *oldlenp,
                  void *newval, size_t newlen, void **context)
{
        int l, len;
        
        if (!table->data || !table->maxlen) 
                return -ENOTDIR;
        
        if (oldval && oldlenp && get_user(oldlenp)) {
                len = get_user(oldlenp);
                l = strlen(table->data);
                if (len > l) len = l;
                if (len >= table->maxlen)
                        len = table->maxlen;
                memcpy_tofs(oldval, table->data, len);
                put_user(0, ((char *) oldval) + len);
                put_user(len, oldlenp);
        }
        if (newval && newlen) {
                len = newlen;
                if (len > table->maxlen)
                        len = table->maxlen;
                memcpy_fromfs(table->data, newval, len);
                if (len == table->maxlen)
                        len--;
                ((char *) table->data)[len] = 0;
        }
        return 0;
}

#endif

#endif
