/*********************************************************************
 *		  
 * Filename:	  irvdd.c
 * Version:	  
 * Description:   Virtual Device Driver
 * Status:	  Experimental.
 * Author:	  Haris Zukanovic <haris@stud.cs.uit.no>
 * Created at:	  Tue Apr 14 12:42:25 1998
 * Modified at:   Wed Jul  8 20:12:38 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998 Haris Zukanovic <haris@stud.cs.uit.no>, 
 *			  Dag Brattli <dagb@cs.uit.no>
 *     All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Haris Zukanovic nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 *
 * Revision History:
 *
 *   14-Apr-98 12:42:33   Haris Zukanovic <haris@stud.cs.uit.no>
 *     Made this header :)
 *
 *   23-Apr-98 Thomas Davis (ratbert@radiks.net)
 *     Ripped to shreds.  :-)
 *
 ********************************************************************/

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/tty.h>
#include <linux/malloc.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <asm/dma.h>

#include "irda.h"
#include "ircompat.h"
#include "irlap_frame.h"
#include "driver.h"
#include "irvdd.h"
#include "irtty.h"
#include "wrapper.h"
#include "qos.h"

extern void irlap_open( struct driver_cb *);
extern void irlap_close( void);

void irvdd_cleanup_driver(struct driver_cb *driver);
void irvdd_bh(void *);

struct driver_cb **irvdds = NULL;

char *irvdd_version = "IrVDD, version 0.1\n";

struct tq_struct irvdd_task;

/* 
 *  Init routines
 */

/*
 * Function irvdd_init (void)
 *
 *    
 *
 */
int irvdd_init(void) 
{
	DEBUG( 4, "--> IrVDD: init\n");

	irvdds = (struct driver_cb **) kmalloc( sizeof(void *) * MAX_IRVDDS,
					      GFP_KERNEL);

	if ( irvdds == NULL) {
		printk( KERN_WARNING "IrVDD: Can't allocate irvdds array!\n");
		return -ENOMEM;
	}

	memset( irvdds, 0, sizeof(void *) * MAX_IRVDDS);

#if 0
#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
	proc_register_dynamic( &proc_irda, &proc_irvdd);
#else
	proc_register( &proc_irda, &proc_irvdd);
#endif
#endif /* CONFIG_PROC_FS */
#endif /* USE_PROC */
#endif

	DEBUG( 4, "IrVDD: init -->\n");
	return 0;
}

void irvdd_cleanup (void)
{
	int i;

	DEBUG( 4, "--> IrVDD: cleanup\n");

	if ( irvdds != NULL) {
		for (i=0; i<MAX_IRVDDS; i++) {
			if (irvdds[i]) {
				if (irvdds[i]->flags & HANDLE_IN_USE) {
					if (irvdds[i]->closed)
						irvdds[i]->closed(irvdds[i]);
				}
				kfree(irvdds[i]);
				irvdds[i] = NULL;
			}
		}
		DEBUG( 4, "freeing master array\n");
		kfree(irvdds);
		irvdds = NULL;
	}

#if 0
#ifdef USE_PROC
#ifdef CONFIG_PROC_FS
	proc_unregister( &proc_irda, proc_irvdd.low_ino);
#endif
#endif
#endif

	DEBUG( 4, "IrVDD: cleanup -->\n");
}

/*
 *  Registration routines
 */

/*
 * Function irvdd_register_init()
 *   
 *    Allocates the data structure for the system to use.
 */
struct driver_cb *irvdd_init_device(void)
{
	struct driver_cb *driver;
	int handle = -1, i;
        
	DEBUG( 4, "--> IrVDD: init_device()\n");

	for (i=0; i<MAX_IRVDDS; i++) {
		driver = irvdds[i];
		if (driver == NULL) {
			DEBUG( 4, "IrVDD: init_device: allocated %d\n", i);
			handle = i;
			break;
		}
		if (!(driver->flags & HANDLE_IN_USE)) {
			DEBUG( 4, "IrVDD: init_device: reusing %d\n", i);
			handle = i;
			break;
		}
	}

	if (handle < 0) {
		DEBUG( 3, "IrVDD: table is full!\n");
		return NULL;
	}

	if (irvdds[handle] == NULL) {
		DEBUG( 4, "IrVDD: allocating entry %d\n", handle);
		driver = kmalloc(sizeof(struct driver_cb), GFP_KERNEL);
		if (driver == NULL) {
			DEBUG( 0, "IrVDD: no memory!\n");
			return NULL;
		}
		memset(driver, 0, sizeof(struct driver_cb));
		irvdds[handle] = driver;
	} else {
		DEBUG( 4, "IrVDD: reusing entry = %d\n", handle);

		driver = irvdds[handle];

		if (driver->magic != DRIVER_CB_MAGIC) {
			DEBUG(3, "IrVDD: entry has bad magic!\n");
			return NULL;
		}
	}

	ASSERT( driver != NULL, return NULL;);

	irlap_init_qos_capabilities( &driver->qos);

	driver->flags |= HANDLE_IN_USE;
	driver->magic = DRIVER_CB_MAGIC;
	driver->irvdd_rev = IRVDD_REV;
	driver->handle = handle;

	return driver;
}

/*
 * Function irvdd_register_device (driver)
 *
 *   Takes the returned driver, and sets everything up.
 *
 */
int irvdd_register_device(struct driver_cb *driver)
{
	int val;
        
	if (driver == NULL || driver->magic != DRIVER_CB_MAGIC) {
		DEBUG( 3, "IrVDD: bad magic detected!\n");
		return(-EINVAL);
	}
        
	if ( driver->ACE.iobase > 0 ) {
		if (check_region(driver->ACE.iobase, 
			 driver->ACE.ioextent) < 0) {
			DEBUG( 0, "IrVDD: can't get ACE iobase of 0x%03x\n",
			       driver->ACE.iobase);
			irvdd_cleanup_driver(driver);
			return -ENODEV;
			request_region( driver->ACE.iobase, 
					driver->ACE.ioextent, 
					driver->driver_name);

			driver->flags |= ACE_IO_ALLOCATED;
		}
	}

	if (driver->SCE.iobase > 0) {
		if (check_region(driver->SCE.iobase, 
				 driver->SCE.ioextent) < 0) {
			DEBUG( 0, "IrVDD: can't get SCE iobase of 0x%03x\n",
			       driver->SCE.iobase);
			irvdd_cleanup_driver(driver);
			return -ENODEV;
		}
		request_region( driver->SCE.iobase, 
				driver->SCE.ioextent, 
				driver->driver_name);

		driver->flags |= SCE_IO_ALLOCATED;
	}

	if (driver->ACE.irq > 0) {
		DEBUG(2, "IrVDD: allocating interrupt %d!\n", driver->ACE.irq);

		/* Allocate fast interrupt */
		val = request_irq( driver->ACE.irq, driver->irq_handler, 
				   SA_INTERRUPT, driver->driver_name, driver);
		if (val) {
			printk("IrVDD: unable to get IRQ %d for %s (irqval=%d).\n",
			       driver->ACE.irq, driver->driver_name, val);
			irvdd_cleanup_driver(driver);
			return -EAGAIN;
		} else {
			DEBUG(2, "IrVDD: allocated interrupt %d!\n", driver->ACE.irq);
			driver->flags |= ACE_IRQ_ALLOCATED;
		}
	}

	if (driver->SCE.irq > 0 && driver->ACE.irq != driver->SCE.irq) {
		DEBUG(2, "IrVDD: allocating SCE interrupt %d!\n", driver->SCE.irq);

		/* Allocate fast interrupt */
		val = request_irq( driver->SCE.irq, driver->irq_handler, 
				   SA_INTERRUPT, driver->driver_name, driver);
		if (val) {
			printk("IrVDD: unable to get SCE IRQ %d for %s (irqval=%d).\n",
			       driver->SCE.irq, driver->driver_name, val);
			irvdd_cleanup_driver(driver);
			return -EAGAIN;
		} else {
			DEBUG(2, "IrVDD: allocated interrupt %d!\n", driver->SCE.irq);
			driver->flags |= SCE_IRQ_ALLOCATED;
		}
	}

	if (driver->SCE.dma > 0) {
		val = request_dma( driver->SCE.dma, driver->driver_name);
	        
		if (val) {
			printk("%s: unable to get DMA channel %d\n", 
			       driver->driver_name, driver->SCE.dma);
			irvdd_cleanup_driver(driver);
			return -EAGAIN;
		} else {
			driver->flags |= DMA_ALLOCATED;
		}
	}

#if 0
	if (driver->driver_bh) {
		DEBUG(3, "IrVDD: registering bottom handler!\n");
		init_bh( IRDRIVER_BH, driver->driver_bh);
	}

	init_bh( IRVDD_BH, irvdd_bh);
#endif

	if (driver->driver_flags & IO_DMA) {
		driver->rx.buff = ( __u8 *) kmalloc( BUF_MAX_LEN, GFP_KERNEL|GFP_DMA);
		driver->tx.buff = ( __u8 *) kmalloc( BUF_MAX_LEN, GFP_KERNEL|GFP_DMA);
	} else {
		driver->rx.buff = ( __u8 *) kmalloc( BUF_MAX_LEN, GFP_KERNEL);
		driver->tx.buff = ( __u8 *) kmalloc( BUF_MAX_LEN, GFP_KERNEL);
	}

	if ( driver->rx.buff == NULL || driver->tx.buff == NULL)   {
		DEBUG( 0, "IrVDD: no space for buffers!\n");
		irvdd_cleanup_driver(driver);
		return -ENOMEM;
	}

	memset(driver->rx.buff, 0, BUF_MAX_LEN);
	memset(driver->tx.buff, 0, BUF_MAX_LEN);

	driver->rx.buffsize=BUF_MAX_LEN;
	driver->tx.buffsize=BUF_MAX_LEN;

	skb_queue_head_init( &driver->rx.list);
	skb_queue_head_init( &driver->tx.list);

	driver->rx.in_frame = FALSE;
	driver->rx.state = OUTSIDE_FRAME;

	printk("IrVDD: IrDA driver %s registered.\n",
	       driver->driver_name);
        
	return(0);
}

/*
 * Function irvdd_unregister_device (driver)
 *
 *    Unregister device with IrVDD
 *
 */
int irvdd_unregister_device(struct driver_cb *driver)
{
	if (driver == NULL) 
		return(-EINVAL);
        
	DEBUG( 4, "IrVDD: removing IrDA driver for %s\n", driver->driver_name);

	irvdd_cleanup_driver(driver);

	return(0);
}

/*
 * Function irvdd_cleanup_driver(device)
 *
 */
void irvdd_cleanup_driver(struct driver_cb *driver)
{
	int ACE_iobase, SCE_iobase;
	struct sk_buff *skb;
	int handle;

	DEBUG( 4, "---> irvdd_cleanup_driver()\n");

	handle = driver->handle;

	if (driver == NULL) {
		DEBUG( 0, "irvdd_cleanup: NULL driver!\n");
		return;
	}

	ACE_iobase = driver->ACE.iobase;
	SCE_iobase = driver->SCE.iobase;

	/* Empty tx_list and rx_list! */

	if ( driver->tx.buff) {
		kfree_s( driver->tx.buff, driver->tx.buffsize);
	}

	if ( driver->rx.buff) {
		kfree_s( driver->rx.buff, driver->rx.buffsize); 
	}

	while (( skb = skb_dequeue( &driver->rx.list)) != NULL) {
		DEV_KFREE_SKB( skb, FREE_READ);  
	}

	while (( skb = skb_dequeue( &driver->tx.list)) != NULL) {
		DEV_KFREE_SKB( skb, FREE_WRITE); 
	}

#if 0
	/* Stop media busy timer */
	del_timer( &device->media_busy_timer);
#endif

	if (driver->flags & DMA_ALLOCATED) {
		free_dma( driver->SCE.dma);
	}

	if (driver->flags & ACE_IRQ_ALLOCATED) {
		free_irq( driver->ACE.irq, driver);
	}

	if (driver->flags & SCE_IRQ_ALLOCATED) {
		free_irq( driver->SCE.irq, driver);
	}

	if (driver->flags & ACE_IO_ALLOCATED) {
		release_region( driver->ACE.iobase, driver->ACE.ioextent);
	}

	if (driver->flags & SCE_IO_ALLOCATED) {
		release_region( driver->SCE.iobase, driver->SCE.ioextent);
	}

	if (driver)
		kfree_s(driver, sizeof(struct driver_cb));

	irvdds[handle] = NULL;
}

/*
 * Function irvdd_open (irtty)
 *
 *    Called by irtty_open() when a new line up through IrDA protocol stack
 *    is to be instantiated. 
 *
 */
int irvdd_open( struct driver_cb *driver)
{
	/* register with the upper layer */
	irlap_open( driver);

	return(0);
}

/* 
 * Function void irvdd_close(struct irtty_cb *irtty)
 *
 *    Closes the connection to IrLAP 
 *
 */
void irvdd_close(struct driver_cb *driver)
{
	irlap_close();
}

/* 
 *   IrLAP interface
 */

/*
 * Function irvdd_queue_xmit (irvdd, skb)
 *
 *    Tries to xmit frame to the underlying active driver, or queues it if
 *    driver is currently busy transmitting.
 * 
 */
inline int irvdd_queue_xmit(struct driver_cb *driver ,struct sk_buff *skb)
{
	/*
	 *  Check if driver exists
	 */
	if ( driver == NULL) {
		DEBUG( 0, "irvdd_xmit(), driver is missing!\n");
		return -1;
	}

	ASSERT( driver->is_tbusy != NULL, return -1;);
	ASSERT( driver->is_media_busy != NULL, return -1;);

	/* 
	 *  If driver is busy transmitting or media is busy then queue
	 *  packet for transmission later
	 */
	if ( driver->is_tbusy(driver) || driver->is_media_busy(driver)) {
		DEBUG( 4, "IrVDD: queuing skb for later transmission!\n");
		skb_queue_tail( &driver->tx.list, skb);

		return 0;
	}

	DEBUG( 6, "IrVDD: irvdd_xmit, sending skb on!\n");
	driver->hard_xmit( driver, skb);

	return 0;
}

/*
 * Function irvdd_change_speed (irvdd, baud)
 *
 *    Change the speed of the currently used low-level driver
 *
 */
inline void irvdd_change_speed( struct driver_cb *driver, int baud)
{
	if (!driver) {
		return;
	}

	driver->change_speed(driver, baud);
}

/*
 * Function irvdd_is_media_busy (irvdd)
 *
 *    Check if media is busy
 *
 */
inline int irvdd_is_media_busy(struct driver_cb *driver)
{
	if (!driver) {
		return -1;
	}

	DEBUG( 4, "irvdd_is_media_busy!\n");

	return( driver->is_media_busy(driver));
}

/*
 * Function irvdd_is_receiving (irvdd)
 *
 *    Check if we are receiving a frame right now!
 *
 */
inline int irvdd_is_receiving(struct driver_cb *driver)
{
	if (!driver) {
		return -1;
	}

	return( driver->is_receiving( driver));
}

void irvdd_mark_bh(struct driver_cb *driver)
{
	DEBUG( 4, "irvdd_mark_bh: processing frames!\n");
        
	irvdd_task.routine = irvdd_bh;
	irvdd_task.data = (void *) driver;
	queue_task(&irvdd_task, &tq_scheduler);
}

void irvdd_bh(void *data)
{
	struct driver_cb *driver = (struct driver_cb *) data;
	struct sk_buff *skb = NULL;

	DEBUG(4, "irvdd_bh: processing frames!\n");

	/* Process all received frames */
	while (( skb = skb_dequeue( &driver->rx.list)) != NULL) {
		DEBUG( 4, "irvdd_bh: delivering frame\n");
#if 0
	{
		int i;
		DEBUG( 3, "skb->len=%d\n", skb->len);
		for( i=0;i<skb->len;i++)
			if (skb->data[i] >31 && skb->data[i]<127) {
				printk("%c", skb->data[i]);
			} else {
				printk("(%02x)", skb->data[i]);
			}
		printk("\n");
	}
#endif
		irlap_input(skb);
	}
        
}
