/*********************************************************************
 *                
 * Filename:      irvdd.h
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Haris Zukanovic <haris@stud.cs.uit.no>
 * Created at:    Tue Apr 14 12:41:57 1998
 * Modified at:   Thu May 21 21:33:32 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998 Haris Zukanovic, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Haris Zukanovic nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifndef IR_VDD
#define IR_VDD

/*  IrLAP should NOT be aware of the differences between underlying FIR MIR
 *  and SIR drivers, therefor we insert a tiny virtual device (vd) layer.
 *  irlap uses this Virtual Device struct when calling underlying ir driver
 *  (FIR MIR or SIR). This layer takes care of correctly routing
 *  hard_xmit(); requests, changing speeds of ir communication at hardware
 *  level,wrapping the packets and taking care of that interrupts from FIR
 *  (MIR) hardware don't go up the whole irda protocol stack.
 *  Struct is filled properly when underlying physical driver is
 *  registering with vd (virtual device) layer and when attaching ir to
 *  it's tty.  The first can be done nicely when running the init_module()
 *  routine of the fir driver. Every ir device uses a serial port to
 *  communicate through when at lower speeds, so a irtty_cb is in irvdd_cb.
 */

/*
 * TAD, April 22, 1998
 * changed to be a TRUE virtual driver; it no longer assumes that you want
 * to use IrTTY.  Allows mixing and matching at the driver level.
 */

#include "irda.h"

/* Space for 4 IR virtual device drivers */

#define MAX_IRVDDS 4

/* check /usr/include/linux/interrupts.h - correct as of v2.0.33 */
#define IRVDD_BH	0x13
#define IRDRIVER_BH	0x14

extern int irvdd_init(void);
extern void irvdd_cleanup(void);

extern int irvdd_queue_xmit(struct driver_cb *, struct sk_buff *);
extern void irvdd_change_speed(struct driver_cb *, int);
extern int irvdd_is_media_busy(struct driver_cb *);
extern int irvdd_is_receiving(struct driver_cb *); 

extern struct driver_cb *irvdd_init_device(void);
extern int irvdd_register_device(struct driver_cb *driver);
extern int irvdd_unregister_device(struct driver_cb *driver);

int irvdd_open( struct driver_cb *driver);
void irvdd_close(struct driver_cb *driver);
void irvdd_mark_bh(struct driver_cb *driver);
#endif /* IR_VDD */

