#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/miscdevice.h>

#include <linux/tty.h>
#include <linux/serial.h>
#include <linux/serial_reg.h> 
#include <linux/ioport.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <linux/malloc.h>

#include "ircompat.h"
#include "irda.h"
#include "smc_ircc.h"
#include "wrapper.h"
#include "driver.h"
#include "irport.h"
#include "irvdd.h"
 
#if 0
static char *rcsid = "$Id: smc_ircc.c,v 1.1.1.1 1998/09/29 02:27:12 root Exp $";
#endif

#undef DUMPBANK
#define PORT_SMC 0x10

static void dumpbank(struct driver_cb *driver, int bankno);
int FIR_receive_mode(struct driver_cb *driver);

void setup_dma(struct driver_cb *driver, char *buffer, int count, int mode)
{
	unsigned long flags;

	save_flags(flags);
	cli();

	disable_dma(driver->SCE.dma);
	clear_dma_ff(driver->SCE.dma);
	set_dma_mode(driver->SCE.dma, mode);
	set_dma_addr(driver->SCE.dma, virt_to_bus(buffer));

	set_dma_count(driver->SCE.dma, count);
	enable_dma(driver->SCE.dma);

	restore_flags(flags);
}

/*
 * Detect IO port
 */

int FIR_detect(struct driver_cb *driver) 
{
	int iobase = driver->SCE.iobase;
	int high, low, chip, version;

	outb((inb(iobase+MASTER) & 0xF0) | 3, iobase+MASTER);

	high = inb_p(iobase+SMSC_ID_HIGH);
	low = inb_p(iobase+SMSC_ID_LOW);
	chip = inb_p(iobase+CHIP_ID);
	version = inb_p(iobase+VERSION_NUMBER);
	driver->SCE.dma = inb_p(iobase+HOST_INTERFACE);

	driver->SCE.irq = (driver->SCE.dma >> 4) & 0x0f;

	driver->SCE.dma &= 0x0F;

	if (high == 0x10 && low == 0xb8 && chip == 0xf1) {
		DEBUG(0, "SMC IrDA controller found; version %d, port 0x%04x, "
		      "dma %d, interrupt %d\n",
		      version, iobase, driver->SCE.dma, driver->SCE.irq);
	} else {
		return -ENODEV;
	}

	/*
	 * Reset the UART.
	 */
	iobase = driver->ACE.iobase;

	outb_p( 0x00, iobase+UART_MCR);
	outb_p((UART_FCR_CLEAR_RCVR | UART_FCR_CLEAR_XMIT), iobase+UART_FCR);
	(void)inb( iobase+UART_RX);
        
	serial_out(driver->SCE.iobase, MASTER, 0);

	return PORT_SMC;
}

void FIR_new_frame(struct driver_cb *driver)
{
	DEBUG(3, "FIR_new_frame!\n");
	driver->rx.state = INSIDE_FRAME;
}

void FIR_eom(struct driver_cb *driver)
{
	struct sk_buff *skb;
	int msgcnt;

	DEBUG(3, "FIR_eom!\n");

	if (driver->direction == RCV) {
		msgcnt = serial_in(driver->SCE.iobase, LCR_B) & 0x03;
		DEBUG( 3,"msgcnt=%d\n", msgcnt);

		driver->rx.count = driver->rx.buffsize - get_dma_residue(driver->SCE.dma) - 4;
		DEBUG( 3, "received %d bytes\n", driver->rx.count);

		skb = dev_alloc_skb(driver->rx.count);
		if (skb == NULL)  {
			printk( KERN_INFO "async_unwrap: memory squeeze, dropping frame.\n");
					/* sl->rx_dropped++; */
			return;
		}

		ALLOC_SKB_MAGIC(skb);
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
		skb->free=1;
#endif
		ASSERT( driver->rx.count > 0, return;);
		/*
		 *  Copy data without CRC
		 */
		DEBUG( 4, "Frame len=%d\n", driver->rx.count);
		skb_put( skb, driver->rx.count);
		memcpy( skb->data, driver->rx.buff, driver->rx.count);
		driver->rx.count = 0;
		skb->mac.raw=skb->data;
		skb_queue_tail( &(driver->rx.list), skb);

		/*
		 *  Now we have something to chew on so inform upper 
		 *  layers.This will schedule a bottom half routine 
		 *  which is responsible for delivering all 
		 *  received frames.
		 */
		irvdd_mark_bh(driver);
		irport_set_media_busy(driver, FALSE);
	}	

	register_bank(driver->SCE.iobase, 1);
	serial_out(driver->SCE.iobase, SCE_CFG_B, serial_in(driver->SCE.iobase, SCE_CFG_B) & ~CFG_B_DMA_ENABLE);

	driver->rx.state = OUTSIDE_FRAME;

	if ((skb = skb_dequeue(&driver->tx.list))) {
		irport_xmit( driver, skb);
	} else {
		FIR_receive_mode(driver);
	}
}

void FIR_receive(struct driver_cb *driver)
{
	int iobase = driver->SCE.iobase;

	DEBUG(3, "FIR_receive!\n");

	register_bank(iobase, 0);

	do {
		*driver->rx.head = inb(iobase);
		driver->rx.ptr++;
		driver->rx.head++;
		driver->rx.count++;
		if (driver->rx.count > 2148) {
			DEBUG( 0, "FIR_receive: large frame!\n");
			break;
		}	
	} while (serial_in(iobase, BSR) & 0x80);

#if 1
	register_bank(driver->SCE.iobase, 0);
	serial_out(driver->SCE.iobase, IER, IER_ACTIVE_FRAME|IER_EOM|IER_FIFO); 
#endif

	serial_out(driver->SCE.iobase, MASTER, MASTER_INT_EN);

	DEBUG( 3, "%d bytes received\n", driver->rx.count);

}

/*
 * Function FIR_receive_mode (void)
 *
 */
int FIR_receive_mode(struct driver_cb *driver) 
{
	int iobase = driver->SCE.iobase;

	DEBUG(3, "FIR_receive_mode!\n");

	memset(driver->rx.buff, 0, driver->rx.buffsize);

	setup_dma(driver, driver->rx.buff, driver->rx.buffsize, DMA_MODE_READ);

	driver->media_busy = FALSE;
	driver->direction = RCV;
	driver->rx.head = 0;

	/* driver->rx.ptr = driver->rx.buff; This can't be right! DB */
	driver->rx.ptr = 0;

	register_bank(iobase, 4);
	serial_out(iobase, IR_CONTROL, (serial_in(iobase, IR_CONTROL) & 0xF0));
	serial_out(iobase, BOF_COUNT_LO, 2);
	serial_out(iobase, BRICK_WALL_CNT_LO, 0);
	serial_out(iobase, BRICK_TX_CNT_HI, 0);
	serial_out(iobase, TX_DATA_SIZE_LO, 0);
	serial_out(iobase, RX_DATA_SIZE_HI, 0);
	serial_out(iobase, RX_DATA_SIZE_LO, 0);

	register_bank(iobase, 0);
	serial_out(iobase, LCR_B, LCR_B_SCE_MODE_RECEIVE | LCR_B_SIP_ENABLE );

	register_bank(iobase, 1);
	serial_out(iobase, SCE_CFG_B, serial_in(iobase, SCE_CFG_B) | CFG_B_DMA_ENABLE);

	register_bank(iobase, 0);

	return 0;
}

void FIR_send(struct driver_cb *driver)
{
	struct sk_buff *skb;

	int iobase = driver->SCE.iobase;
	int cnt, fifo_size=driver->SCE.fifo_size;

	DEBUG(3, "FIR_send: fifo_size=%d, count=%d, ptr=%d\n", fifo_size, driver->tx.count, driver->tx.ptr);

	register_bank(iobase, 0);
	do {
		cnt = driver->tx.count - driver->tx.ptr;
		if (cnt <= 0) {
			serial_out(iobase, LCR_A, (serial_in(iobase, LCR_A) | LCR_A_DATA_DONE));
			DEBUG( 3, "FIR_send: finished with frame\n"); 
			DEBUG( 3, "FIR_send: queue len: %d\n", 
			       skb_queue_len( &driver->tx.list));
			irport_set_media_busy( driver, FALSE);
			if ((skb = skb_dequeue(&driver->tx.list))) {
				irport_xmit( driver, skb);
#if 0
			} else {
				FIR_receive_mode(driver);
#endif
			}
			dumpbank(driver, 0);
			break;
		}	        
		/* Transmit next byte */
		outb( *driver->tx.head, iobase);
		driver->tx.ptr++;
		driver->tx.head++;
	} while (--fifo_size > 0);

	serial_out(driver->SCE.iobase, MASTER, MASTER_INT_EN);

	DEBUG( 3, "left %d bytes free in fifo; %d sent of %d, cnt=%d\n", 
	       fifo_size, driver->tx.ptr, driver->tx.count, cnt);
}

void FIR_setspeed(struct driver_cb *driver, int baud)
{
	int byte;
	int ir_mode, select=0, fast=0;
	int iobase = driver->SCE.iobase;

	DEBUG(3, "--> FIR_setspeed()\n");

	serial_out(driver->ACE.iobase, 4, 0);	/* disable interrupts from the ACE */

	serial_out(iobase, MASTER, MASTER_RESET);

	register_bank(iobase, 0);
	serial_out(iobase, IER, 0); /* shutdown ALL interrupts from the SCE */

	switch (baud) {
	case 576000:
		ir_mode = CFG_A_IRDA_HDLC;
		select = 0;
		fast = 0;
		DEBUG(0, "FIR_setspeed: handling baud of 576000\n");
		break;
	case 1152000:
		ir_mode = CFG_A_IRDA_HDLC;
		select = SELECT_1152;
		fast = 0;
		DEBUG(0, "FIR_setspeed: handling baud of 1152000\n");
		break;
	case 4000000:
		ir_mode = CFG_A_IRDA_4PPM;
		select = 0;
		fast = LCR_A_FAST;
		DEBUG(0, "FIR_setspeed: handling baud of 4000000\n");
		break;
	default:
		ir_mode = CFG_A_IRDA_4PPM;
		select = 0;
		fast = LCR_A_FAST;
		DEBUG(0, "FIR_setspeed: unknown baud rate of %d\n", baud);
		break;
	}

	register_bank(iobase, 1);
        
	serial_out(iobase, SCE_CFG_A, (serial_in(iobase, SCE_CFG_A) & 0x87) | ir_mode );

	serial_out(iobase, SCE_CFG_B, (serial_in(iobase, SCE_CFG_B) & 0x3f) | CFG_B_MUX_IR );

	byte = serial_in(iobase, FIFO_THRESHOLD);
	serial_out(iobase, FIFO_THRESHOLD, 64);

	register_bank(iobase, 4);

	serial_out(iobase, IR_CONTROL, 
		   (serial_in(iobase, IR_CONTROL) & 0x30) | select | CRC_SELECT );

	register_bank(iobase, 0);

	serial_out(iobase, LCR_A, fast);
        
	DEBUG(3, "FIR_setspeed() -->\n");
}

void FIR_shutdown(struct driver_cb *driver)
{
	DEBUG(3, "FIR_shutdown()\n");

	register_bank(driver->SCE.iobase, 0);
	serial_out(driver->SCE.iobase, IER, 0); /* shutdown ALL interrupts from the SCE */

	serial_out(driver->SCE.iobase, MASTER, MASTER_RESET);

	register_bank(driver->SCE.iobase, 1);
	serial_out(driver->SCE.iobase, SCE_CFG_A, CFG_A_IRDA_SIR_A | CFG_A_TX_POLARITY);
	serial_out(driver->SCE.iobase, SCE_CFG_B, CFG_B_MUX_IR);

	DEBUG(3, "FIR_shutdown called!\n");
}

/*
 * Function FIR_xmit (void)
 *
 *    Transmits the current frame until FIFO is full, then
 *    waits until the next transmitt interrupt, and continues until the
 *    frame is transmited.
 */
int FIR_xmit(struct driver_cb *driver) 
{
	int iobase;

	DEBUG(3, "FIR_xmit: driver->tx.count=%d, ptr=%d\n", driver->tx.count, driver->tx.ptr);

	iobase = driver->SCE.iobase;
        
	setup_dma(driver, driver->tx.buff, driver->tx.count, DMA_MODE_WRITE);

	driver->media_busy = TRUE;
	driver->direction = XMIT;

	serial_out(driver->ACE.iobase, 4, 0x08);

	register_bank(iobase, 4);
	serial_out(iobase, IR_CONTROL, (serial_in(iobase, IR_CONTROL) & 0xF0));
	serial_out(iobase, BOF_COUNT_LO, driver->qos.additional_bofs_value & 0xFF);
	serial_out(iobase, BRICK_WALL_CNT_LO, 0);
#if 0
	serial_out(iobase, BRICK_TX_CNT_HI, (driver->tx.count >> 8) & 0x0F);
	serial_out(iobase, TX_DATA_SIZE_LO, driver->tx.count & 0xFF);
#else
	serial_out(iobase, BRICK_TX_CNT_HI, 0);
	serial_out(iobase, TX_DATA_SIZE_LO, 0);
#endif

	register_bank(iobase, 1);
	serial_out(iobase, SCE_CFG_B, serial_in(iobase, SCE_CFG_B) | CFG_B_DMA_ENABLE);

	register_bank(iobase, 0);

	serial_out(iobase, IER, IER_ACTIVE_FRAME|IER_EOM); 
	serial_out(iobase, LCR_B, LCR_B_SCE_MODE_TRANSMIT | LCR_B_SIP_ENABLE);

	serial_out(iobase, MASTER, MASTER_INT_EN);

	return 0;
}

void FIR_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	struct driver_cb *driver = (struct driver_cb *) dev_id;

	int iir;
#if 0
	int msgcnt;
#endif

	serial_out(driver->SCE.iobase, MASTER, 0);

	register_bank(driver->SCE.iobase, 0);

	iir = serial_in(driver->SCE.iobase, IIR);
	DEBUG(4, "ircc_interrupt: iir=0x%02x\n", iir);

	serial_out(driver->SCE.iobase, IER, 0);

	if (iir & IIR_ACTIVE_FRAME) {
		FIR_new_frame(driver);
	}

	if (iir & IIR_EOM) {
		FIR_eom(driver);
	}

#if 0
	if (iir & IIR_RAW_MODE) {
		DEBUG(, "ircc_interrupt: IR RAW mode not supported!\n");
	}
#endif

#if 0
	if (iir & IIR_FIFO) {
		msgcnt = serial_in(driver->SCE.iobase, LCR_B) & 0x03;
		DEBUG( 3,"msgcnt=%d\n", msgcnt);
		serial_out(driver->SCE.iobase, LSR, msgcnt);
		/* iir = (serial_in(driver->SCE.iobase, LCR_B) >> 6) & 0x03; */
		iir = serial_in(driver->SCE.iobase, LSR);
		DEBUG(0, "FIR_interrupt: FIFO wants attention, lsr=0x%02x!\n", iir);
	        
		if (driver->direction == XMIT) {
			FIR_send(driver);
		} else {
			FIR_receive(driver);
		}
	}
#endif

	register_bank(driver->SCE.iobase, 0);

	serial_out(driver->SCE.iobase, IER, IER_ACTIVE_FRAME|IER_EOM);
	serial_out(driver->SCE.iobase, MASTER, MASTER_INT_EN);

	DEBUG(4, "ircc_interrupt: end!\n");

	return;
}

static void dumpbank(struct driver_cb *driver, int bankno)
{
	int i;
	int iobase = driver->SCE.iobase;

	register_bank(iobase, bankno);

	for (i=0; i<8; i++) {
		DEBUG(3, "bank %d, r%d=0x%02x\n", bankno, i, serial_in(iobase, i));
	}
}
